@@ -10,13 +10,9 @@ use lazy_static::lazy_static;
 
 use maplit::hashmap;
 
-#[cfg(feature = "compile")]
-use serde::{Serialize, Deserialize};
-
 use crate::RefTVal;
 
 #[derive(Clone)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct HashableMap<K, V>
 where
     K: Hash + Ord,
