@@ -21,7 +21,7 @@ project's `Cargo.toml`:
 
 ```
 [dependencies]
-umbra_lang = "0.19"
+umbra_lang = "0.21.0"
 ```
 
 To use the library, call the desired `run*()` functions (documented below). For
@@ -31,71 +31,58 @@ example:
 let env = Env::prelude();
 let (vars, val) = run_path("script.um", &env, true);
 ```
-
-# Crate Features
-
-* **readline** - *Default*, enables use of rustyline for input in
-`run_interactive()`.
-* **compile** - Enables the ability to serialize ASTs and deserialize them for
-execution. This is generally untested and currently not any faster than normal.
 */
 
 #![allow(incomplete_features)]
-#![feature(const_generics)]
+#![feature(adt_const_params)]
 
 #![feature(iter_intersperse)]
 
 #![deny(rust_2018_idioms)]
 
 #![warn(clippy::pedantic)]
-#![allow(clippy::needless_lifetimes)]
 #![allow(clippy::upper_case_acronyms)]
 #![allow(clippy::similar_names)]
 #![allow(clippy::needless_pass_by_value)]
-#![allow(clippy::match_wildcard_for_single_variants)]
 #![allow(clippy::too_many_lines)]
-#![allow(clippy::clippy::option_if_let_else)]
 #![allow(clippy::module_name_repetitions)]
-#![allow(clippy::wildcard_imports)]
 #![allow(clippy::type_complexity)]
+#![allow(clippy::unnested_or_patterns)]
+#![allow(clippy::if_then_panic)]
 
+#[cfg(not(target_env = "msvc"))]
+#[cfg(feature = "jemallocator")]
 #[global_allocator]
 static ALLOC: jemallocator::Jemalloc = jemallocator::Jemalloc;
 
-use std::{fmt, fs};
+use std::{fmt, fs, io};
 use std::collections::{HashMap, hash_map::DefaultHasher};
 use std::hash::{Hash, Hasher};
 use std::path::{Path, PathBuf};
 use std::ffi::OsStr;
 
+#[cfg(target_env = "msvc")]
+use std::io::{Write, BufRead};
+
 use lazy_static::lazy_static;
 use maplit::hashmap;
 
-#[cfg(feature = "readline")]
-use rustyline as rl;
-#[cfg(not(feature = "readline"))]
-use std::io::{self, BufRead, Write};
-
-#[cfg(feature = "compile")]
-use serde::{Serialize, Deserialize, Serializer, Deserializer, ser, de};
-
-#[cfg(feature = "compile")]
-use std::fmt::Display;
-
 mod hashablemap;
 use hashablemap::HashableMap;
 mod threadhandle;
-pub use threadhandle::ThreadHandle;
+pub use threadhandle::{ThreadConfig, ThreadHandle};
 mod thrio;
+mod internaldata;
+use internaldata::InternalData;
 
 mod parsing;
-use parsing::*;
+use parsing::{ENUM_VALUE, Expr, Exprs, Line, Lines, Tokens, VecMap, unescape, match_pat, parse_number};
 pub use parsing::{Pos, RefTVal, TVal, Token, AST, Callable, FnArgs, FnReturn, TokenType, Type, Value};
 mod runnable;
 pub use runnable::*;
 
 mod ops;
-use ops::{Op, OpTrait};
+use ops::{Op, OpTrait, Precedence};
 mod umcore;
 mod umstd;
 mod ummod;
@@ -127,25 +114,27 @@ impl fmt::Display for Error {
                 Some(pos) => write!(f, "Script Error at {}: {}", pos, m),
                 None => write!(f, "Script Error: {}", m),
             },
-            Error::Control(m, v) => write!(f, "Control Error: {}: {:?}", m, v),
+            Error::Control(m, v) => {
+                match v {
+                    Some(v) if m == "panic" => {
+                        if let Value::Enum(_, e) = &v.val {
+                            if let Value::List(l) = e.1[0].clone_out().val {
+                                if let Some(pos) = Pos::from_value(&l[0].clone_out().val) {
+                                    if let Value::String(s) = l[1].clone_out().val {
+                                        return write!(f, "Panic at {}: {}", pos, s);
+                                    }
+                                }
+                            }
+                        }
+                        write!(f, "Unknown panic: {}", v.val)
+                    },
+                    _ => write!(f, "Control Error: {}: {:?}", m, v),
+                }
+            },
             Error::Custom(v) => write!(f, "Other Error: {}", v.val),
         }
     }
 }
-#[cfg(feature = "compile")]
-impl ser::Error for Error {
-    fn custom<T: Display>(msg: T) -> Self
-    {
-        Error::Script(msg.to_string(), None)
-    }
-}
-#[cfg(feature = "compile")]
-impl de::Error for Error {
-    fn custom<T: Display>(msg: T) -> Self
-    {
-        Error::Script(msg.to_string(), None)
-    }
-}
 impl std::error::Error for Error {}
 
 /**
@@ -210,6 +199,8 @@ pub struct Env<'a> {
 }
 lazy_static! {
     static ref CORE: Env<'static> = {
+        log::debug!("Initializing core...");
+
         let mut env = Env::from(hashmap!{
             "_".into() => Value::Type(Type::any(), HashableMap::arc()).into(),
         });
@@ -222,6 +213,8 @@ lazy_static! {
     static ref PRELUDE: Env<'static> = {
         let mut env = Env::child(&CORE);
 
+        log::debug!("Initializing prelude...");
+
         // Import Umbra STD
         umstd::init(&mut env);
 
@@ -279,7 +272,7 @@ impl<'a> Env<'a> {
     child.
     */
     #[must_use]
-    pub fn flatten<'b>(old: &'b Env<'a>) -> Env<'a> {
+    pub fn flatten(old: &Env<'a>) -> Env<'a> {
         Env::from(
             Self::iter_flatten(old)
                 .map(|(k, v)| {
@@ -331,11 +324,13 @@ impl<'a> Env<'a> {
     Computes the changed vars in the new env `nenv` that exist in the outside
     env `self`.
     */
-    pub fn diff(&self, nenv: &'a mut Env<'a>) -> HashMap<String, RefTVal> {
-        let outer = match self.parent {
-            Some(p) => p,
-            None => self,
-        };
+    #[must_use]
+    pub fn diff(&self, mut nenv: Env<'a>) -> HashMap<String, RefTVal> {
+        // let outer = match self.parent {
+        //     Some(p) => p,
+        //     None => self,
+        // };
+        let outer = self;
         let mut vars = hashmap!{};
         for (k, v) in nenv.vars.drain() {
             if let Some(ov) = outer.get(&k) {
@@ -372,25 +367,6 @@ impl<'a> fmt::Debug for Env<'a> {
         write!(f, "Env {{ parent: {}, vars: {{{:?}}} }}", parent, self.vars)
     }
 }
-#[cfg(feature = "compile")]
-impl<'a> Serialize for Env<'a> {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where
-        S: Serializer
-    {
-        let nenv = Env::flatten(self);
-        nenv.vars.serialize(serializer)
-    }
-}
-#[cfg(feature = "compile")]
-impl<'a, 'de> Deserialize<'de> for Env<'a> {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: Deserializer<'de>
-    {
-        Ok(Env::from(HashMap::deserialize(deserializer)?))
-    }
-}
 
 /**
 Splits, tokenizes, parses, and constructs a Runnable from `script`. Generally
@@ -404,35 +380,28 @@ when greater than 0. When greater than 1 it will also print `Token`s.
 # Errors
 Will return `Err` if `AST::parse()` resulted in an error for any expressions.
 */
-pub fn compile<'a>(scriptname: &str, script: &str, env: &'a Env<'a>, pos: Option<Pos>, printout: usize) -> Result<Runnable<'a>, Error> {
+pub fn compile<'a>(scriptname: &str, script: &str, env: &'a Env<'a>, pos: Option<Pos>) -> Result<Runnable<'a>, Error> {
     // Split script string into lines while preserving containers
     let lines: Vec<Line> = Lines::split(scriptname, script, pos)
         // Expression debug print
         .inspect(|l| {
-            if printout > 0 && !l.data.starts_with("//") {
-                eprintln!("{}", l);
+            if !l.data.starts_with("//") {
+                log::trace!("{}", l);
             }
         }).collect();
 
     // Convert the line into a series of expressions
-    let exprs: Vec<Expr> = lines.iter().flat_map(|l| Exprs::split(&l)).collect();
+    let exprs: Vec<Expr> = lines.iter().flat_map(Exprs::split).collect();
 
     // Convert the expr into a series of token streams
-    let mut tokens: Vec<Tokens> = exprs.iter().map(|e| Tokens::tokenize(&e))
-        // Token debug prints
-        .inspect(|ts| {
-            if printout > 1 {
-                eprint!("    tokens: [");
-                for t in ts.clone() {
-                    eprint!("{}, ", t);
-                }
-                eprintln!("]");
-            }
-        }).collect();
+    let mut tokens: Vec<Tokens> = exprs.iter()
+        .map(Tokens::tokenize)
+        // Token debug print moved to AST::parse()
+        .collect();
 
     // Parse the token streams into ASTs
     let asts: Result<Vec<AST>, Error> = tokens.drain(0..).map(|ts| {
-            AST::parse(ts, &env)
+            AST::parse(ts, env)
         }).collect();
 
     let mut shash = DefaultHasher::new();
@@ -463,13 +432,13 @@ Debug output can be toggled with printout whose first element will be sent to
 `compile` and whose second element will determine whether or not to print the
 value of each top-level expression in the script.
 */
-pub fn run<'a>(scriptname: &str, script: &str, env: &'a Env<'a>, pos: Option<Pos>, printout: (usize, bool)) -> FnReturn {
+pub fn run(scriptname: &str, script: &str, env: &Env<'_>, pos: Option<Pos>, printout: bool) -> FnReturn {
     // Compile the script
-    let run = match compile(scriptname, script, env, pos, printout.0) {
+    let run = match compile(scriptname, script, env, pos) {
         Ok(r) => r,
         Err(m) => {
-            if printout.1 {
-                eprintln!("{}\n", m);
+            if printout {
+                log::info!("{}\n", m);
             }
             return (None, Err(m));
         },
@@ -494,10 +463,10 @@ pub fn run<'a>(scriptname: &str, script: &str, env: &'a Env<'a>, pos: Option<Pos
                     }
 
                     // Expression value debug print
-                    if printout.1 {
+                    if printout {
                         match val {
-                            Ok(ref v) => eprintln!("==> {}\n", v),
-                            Err(ref m) => eprintln!("{}\n", m),
+                            Ok(ref v) => log::info!("==> {}\n", v),
+                            Err(ref m) => log::error!("{}\n", m),
                         }
                     }
 
@@ -523,7 +492,7 @@ file doesn't exist `.um` will be appended.
 If `run_main` is true and the script defines a function named `main` then that
 function will be run after the script body is done.
 */
-pub fn run_path<'a, P: AsRef<Path>>(path: &P, env: &'a Env<'a>, run_main: bool) -> FnReturn {
+pub fn run_path<P: AsRef<Path>>(path: &P, env: &Env<'_>, run_main: bool) -> FnReturn {
     // Correct path for modules
     let path = path.as_ref();
     let path: PathBuf = if path.is_dir() {
@@ -540,7 +509,7 @@ pub fn run_path<'a, P: AsRef<Path>>(path: &P, env: &'a Env<'a>, run_main: bool)
         Err(m) => return (None, Err(Error::Argument(format!("Failed to load {}: {}", path.display(), m)))),
     };
     // Run script
-    let (vars, val) = match run(&path.to_string_lossy(), &script, env, None, (0, false)) {
+    let (vars, val) = match run(&path.to_string_lossy(), &script, env, None, false) {
         (vars, Ok(v)) => (vars, v),
         (vars, Err(m)) => return (vars, Err(m)),
     };
@@ -569,72 +538,123 @@ pub fn run_path<'a, P: AsRef<Path>>(path: &P, env: &'a Env<'a>, run_main: bool)
     (vars, Ok(val))
 }
 
-#[cfg(feature = "readline")]
-fn run_interactive_readline<'a>(name: &str, env: &'a Env<'a>) {
-    let mut nenv = Env::child(env);
-    let mut pos = Pos::start(&format!("<stdin/{}>", name));
-    let mut ed = rl::Editor::<()>::new();
-    loop {
-        // Read a line from input
-        use rl::error::ReadlineError;
-        match ed.readline(&format!("{}:{}> ", name, pos.line)) {
-            Ok(line) => {
-                if line.trim().is_empty() {
-                    eprintln!();
-                    continue;
-                }
-                ed.add_history_entry(line.clone());
-
-                // Run the line
-                let (vars, _) = run(&format!("<stdin/{}>", name), &line, &nenv, Some(pos.clone()), (0, true));
-                if let Some(vars) = vars {
-                    nenv.update(vars);
-                }
-                pos.line += 1;
-            },
-            Err(ReadlineError::Interrupted) => {
-                eprintln!("^C");
-                return
-            },
-            Err(ReadlineError::Eof) => continue,
-            Err(m) => {
-                eprintln!("Error: {:?}", m);
-                continue;
-            },
+#[cfg(not(target_env = "msvc"))]
+struct EmptyCompleter;
+#[cfg(not(target_env = "msvc"))]
+impl liner::Completer for EmptyCompleter {
+    fn completions(&mut self, _start: &str) -> Vec<String> {
+        vec![]
+    }
+}
+#[cfg(not(target_env = "msvc"))]
+struct EnvCompleter<'a> (Env<'a>);
+#[cfg(not(target_env = "msvc"))]
+impl<'a> liner::Completer for EnvCompleter<'a> {
+    fn completions(&mut self, start: &str) -> Vec<String> {
+        if start.is_empty() {
+            Env::flatten(&self.0).vars.keys()
+                .cloned()
+                .collect()
+        } else {
+            Env::flatten(&self.0).vars.keys()
+                .filter(|k| k.starts_with(start))
+                .cloned()
+                .collect()
         }
     }
 }
+
+/// Init vars specific to the interactive REPL
+fn interactive_init(env: &mut Env<'_>) {
+    env.set("help", &Value::String(
+"\n
+Welcome to the Umbra interpreter!
+
+Try pressing TAB to see variables that are available in the current
+environment. TAB can also be used to complete initial variable names.
+
+For examples of language usage take a look at the test module, and for
+instructions on usage as a library check out https://docs.rs/umbra-lang/
+
+To exit, either press Ctrl-C or run `exit()` without quotes.
+\n".into()).into());
+}
+
 /**
 Start an interactive interpreter session (REPL) in the given environment.
 
 The prompt uses `name` to make it easier to differentiate debug points.
-*/
-pub fn run_interactive<'a>(name: &str, env: &'a Env<'a>) {
-    #[cfg(feature = "readline")]
-    return run_interactive_readline(name, env);
 
-    #[cfg(not(feature = "readline"))]
+# Panics
+Will panic if IO fails to flush or lock.
+*/
+pub fn run_interactive(name: &str, env: &Env<'_>) {
+    #[cfg(target_env = "msvc")]
+    return run_interactive_basic(name, env);
+
+    #[cfg(not(target_env = "msvc"))]
     {
         let mut nenv = Env::child(env);
+        interactive_init(&mut nenv);
         let mut pos = Pos::start(&format!("<stdin/{}>", name));
-        let mut line = String::new();
+        let mut ctx = liner::Context::new();
         loop {
             // Read a line from input
-            line.clear();
-            eprint!("{}:{}> ", name, pos.line);
-            io::stdout().flush().unwrap();
-            io::stdin().lock().read_line(&mut line).unwrap();
-            if line.trim().is_empty() {
-                eprintln!();
-                continue;
-            }
+            match ctx.read_line(&format!("{}:{}> ", name, pos.line), None, &mut EnvCompleter(nenv.clone())) {
+                Ok(line) => {
+                    if line.trim().is_empty() {
+                        eprintln!();
+                        continue;
+                    }
+                    if let Err(m) = ctx.history.push(line.clone().into()) {
+                        log::error!("Failed to write line to history: {}", m);
+                        break;
+                    };
 
-            // Run the line
-            let (vars, _) = run(&format!("<stdin/{}>", name), &line, &nenv, Some(pos.clone()), (0, true));
-            if let Some(vars) = vars {
-                nenv.update(vars);
+                    // Run the line
+                    let (vars, _) = run(&format!("<stdin/{}>", name), &line, &nenv, Some(pos.clone()), true);
+                    if let Some(vars) = vars {
+                        nenv.update(vars);
+                    }
+                    pos.line += 1;
+                },
+                Err(e @ io::Error { .. }) if e.kind() == io::ErrorKind::Interrupted => {
+                    eprintln!("^C");
+                    return;
+                },
+                Err(e @ io::Error { .. }) if e.kind() == io::ErrorKind::UnexpectedEof => {
+                    continue;
+                },
+                Err(m) => {
+                    log::error!("Error: {:?}", m);
+                    continue;
+                },
             }
-            pos.line += 1;
         }
     }
 }
+#[cfg(target_env = "msvc")]
+fn run_interactive_basic(name: &str, env: &Env<'_>) {
+    let mut nenv = Env::child(env);
+    interactive_init(&mut nenv);
+    let mut pos = Pos::start(&format!("<stdin/{}", name));
+    let mut line = String::new();
+    loop {
+        // Read a line from input
+        line.clear();
+        eprint!("{}:{}> ", name, pos.line);
+        io::stdout().flush().unwrap();
+        io::stdin().lock().read_line(&mut line).unwrap();
+        if line.trim().is_empty() {
+            eprintln!();
+            continue;
+        }
+
+        // Run the line
+        let (vars, _) = run(&format!("<stdin/{}", name), &line, &nenv, Some(pos.clone()), true);
+        if let Some(vars) = vars {
+            nenv.update(vars);
+        }
+        pos.line += 1;
+    }
+}
