@@ -1,18 +1,11 @@
 use std::collections::HashMap;
-use std::str::FromStr;
 use std::cmp;
 use std::sync::Arc;
 
-use num::{BigInt, BigRational, FromPrimitive, One, Zero, ToPrimitive};
-
-#[cfg(feature = "compile")]
-use serde::{Serialize, Deserialize};
-
-use crate::{Env, Error, Exprs, FnArgs, FnReturn, Lines, Op, OpTrait, Pos, TVal, RefTVal, Token, TokenType, Tokens, Type, Value, VecMap, umcore, unescape};
+use crate::{Env, Error, Exprs, FnArgs, FnReturn, Lines, Op, OpTrait, Pos, RefTVal, Token, TokenType, Tokens, Type, Value, VecMap, Precedence, Callable, umcore, unescape, parse_number};
 
 /// The parsed representation of a script expression.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum AST {
     /// A function call that accepts tokens as arguments.
     Macro {
@@ -45,7 +38,7 @@ impl AST {
 
             let st = match &e.1.get(0) {
                 Some(rv) => {
-                    if let TVal { val: Value::Struct(st), .. } = rv.clone_out() {
+                    if let Value::Struct(st) = rv.clone_out().val {
                         st
                     } else {
                         return None
@@ -57,49 +50,49 @@ impl AST {
                 "Macro" => return Some(AST::Macro {
                     tokens: match st.get("tokens") {
                         Some(rv) => {
-                            if let TVal { val: Value::List(l), .. } = rv.clone_out() {
+                            if let Value::List(l) = rv.clone_out().val {
                                 l.iter().map(|tv| Token::from_value(&tv.clone_out().val)).collect::<Option<Vec<Token>>>()?
                             } else {
                                 return None
                             }
                         },
-                        _ => return None,
+                        None => return None,
                     },
                 }),
                 "Container" => return Some(AST::Container {
                     token: match st.get("token") {
                         Some(tv) => Token::from_value(&tv.clone_out().val)?,
-                        _ => return None,
+                        None => return None,
                     },
                     children: match st.get("children") {
                         Some(rv) => {
-                            if let TVal { val: Value::List(l), ..} = rv.clone_out() {
+                            if let Value::List(l) = rv.clone_out().val {
                                 l.iter().map(|tv| AST::from_value(&tv.clone_out().val)).collect::<Option<Vec<AST>>>()?
                             } else {
                                 return None
                             }
                         },
-                        _ => return None,
+                        None => return None,
                     },
                 }),
                 "Operator" => return Some(AST::Operator {
                     token: match st.get("token") {
                         Some(tv) => Token::from_value(&tv.clone_out().val)?,
-                        _ => return None,
+                        None => return None,
                     },
                     lhs: match st.get("lhs") {
                         Some(tv) => Box::new(AST::from_value(&tv.clone_out().val)?),
-                        _ => return None,
+                        None => return None,
                     },
                     rhs: match st.get("rhs") {
                         Some(tv) => Box::new(AST::from_value(&tv.clone_out().val)?),
-                        _ => return None,
+                        None => return None,
                     },
                 }),
                 "Value" => return Some(AST::Value {
                     token: match st.get("token") {
                         Some(tv) => Token::from_value(&tv.clone_out().val)?,
-                        _ => return None,
+                        None => return None,
                     },
                 }),
                 _ => {},
@@ -174,12 +167,14 @@ impl AST {
     /// Will return `Err` if `parse()` on children resulted in an error.
     /// An internal function will error if it's unable to reduce adjacent ASTs
     /// into a single AST.
-    pub fn parse<'a, I>(tokens: I, env: &Env<'a>) -> Result<AST, Error>
+    pub fn parse<I>(tokens: I, env: &Env<'_>) -> Result<AST, Error>
     where
         I: IntoIterator<Item = Token>
     {
         // Map tokens into flat ASTs
         let tokens: Vec<Token> = tokens.into_iter().collect();
+        log::trace!("tokens: {:?}", tokens);
+
         let mut asts: Vec<Result<AST, Error>> = tokens.iter().enumerate()
             .fold(vec![], |mut acc: Vec<Result<AST, Error>>, (i, t)| {
                 if let Some(Ok(AST::Macro { .. })) = acc.last() {
@@ -189,7 +184,7 @@ impl AST {
                 match t.ttype {
                     TokenType::Identifier => {
                         if let Some(rv) = env.get(&t.data) {
-                            if let TVal { val: Value::Function { args, .. }, .. } = rv.clone_out() {
+                            if let Value::Function { args, .. } = rv.clone_out().val {
                                 if args == umcore::_macro_args()
                                     || (args.contains_key("env") && args.contains_key("pos") && args.contains_key("tokens"))
                                 {
@@ -254,124 +249,148 @@ impl AST {
         AST::group_precedence(asts?)
     }
     fn group_precedence(mut asts: Vec<AST>) -> Result<AST, Error> {
-        // TODO actually determine precedence
+        let mut next_prec = 0;
         while asts.len() > 1 {
             let mut i: usize = 0;
             let mut unchanged = 0;
-            while i < asts.len() {
-                match asts[i].clone() {
-                    AST::Operator { token, lhs, rhs } if (token.data == "." || token.data == "@") && *lhs == AST::Empty && *rhs == AST::Empty => {
-                        let lhs = if i > 0 {
-                            asts.remove(i-1)
-                        } else {
-                            i += 1;
-                            AST::Empty
-                        };
-                        if i > 0 {
-                            let _op = asts.remove(i-1); // index changed from previous remove
-                        } else {
-                            let _op = asts.remove(i);
-                            i += 1;
-                        }
-                        let rhs = if i > 0 && i <= asts.len() {
-                            asts.remove(i-1)
-                        } else {
-                            AST::Empty
-                        };
+            while i <= asts.len() && asts.len() > 1 {
+                if i == asts.len() {
+                    i = 0;
+                    unchanged += 1;
+                    next_prec += 1;
+                }
+                // log::trace!("i {} unc {} prec {}", i, unchanged, next_prec);
 
-                        unchanged = 0;
-                        asts.insert(i-1, AST::Operator {
-                            token,
-                            lhs: Box::new(lhs),
-                            rhs: Box::new(rhs),
-                        });
-                    },
-                    AST::Container { token, children } if i > 0 => {
-                        match &asts[i-1] {
-                            AST::Operator { rhs, .. } if **rhs == AST::Empty =>  {
-                                unchanged = 0;
-                            },
-                            _ => {
-                                let lhs = asts.remove(i-1);
-                                let _cont = asts.remove(i-1); // index changed from previous remove
+                match asts[i].clone() {
+                    // Reduce operators
+                    AST::Operator { token, lhs, rhs } if *lhs == AST::Empty && *rhs == AST::Empty => {
+                        if let Some(op) = Op::new(&token) {
+                            if Op::prec(op) as usize <= next_prec {
+                                log::trace!("Reducing op: {}", token);
+                                // log::trace!("asts: {:?}", asts);
+                                let lhs = if i > 0 {
+                                    asts.remove(i-1)
+                                } else {
+                                    i += 1;
+                                    AST::Empty
+                                };
+                                if i > 0 {
+                                    let _op = asts.remove(i-1); // index changed from previous remove
+                                } else {
+                                    let _op = asts.remove(i);
+                                    i += 1;
+                                }
+                                let rhs = if i > 0 && i <= asts.len() {
+                                    asts.remove(i-1)
+                                } else {
+                                    AST::Empty
+                                };
 
                                 unchanged = 0;
                                 asts.insert(i-1, AST::Operator {
-                                    token: token.clone(),
+                                    token,
                                     lhs: Box::new(lhs),
-                                    rhs: Box::new(AST::Container {
-                                        token: Token {
-                                            ttype: TokenType::Symbol, // Indicate function call with Symbol
-                                            ..token
-                                        },
-                                        children,
-                                    }),
+                                    rhs: Box::new(rhs),
                                 });
+                            }
+                        }
+                    },
+                    // Reduce containers if they're for function calls, indexes, or constructors
+                    AST::Container { token, children } if i > 0 && !token.data.starts_with("//") => {
+                        if Precedence::L1 as usize <= next_prec {
+                            log::trace!("Reducing container: {}", token);
+                            // log::trace!("asts: {:?}", asts);
+                            match &asts[i-1] {
+                                AST::Operator { rhs, .. } if **rhs == AST::Empty =>  {
+                                    // unchanged = 0;
+                                },
+                                _ => {
+                                    let lhs = asts.remove(i-1);
+                                    let _cont = asts.remove(i-1); // index changed from previous remove
+
+                                    unchanged = 0;
+                                    asts.insert(i-1, AST::Operator {
+                                        token: token.clone(),
+                                        lhs: Box::new(lhs),
+                                        rhs: Box::new(AST::Container {
+                                            token: Token {
+                                                ttype: TokenType::Symbol, // Indicate function call with Symbol
+                                                ..token
+                                            },
+                                            children,
+                                        }),
+                                    });
+                                },
+                            }
+                        }
+                    },
+
+                    // Remove comments
+                    AST::Container { token, .. } if token.data.starts_with("//") => {
+                        asts.remove(i);
+                        unchanged = 0;
+                    },
+
+                    AST::Macro { .. } | AST::Container { .. } => unchanged = 0,
+                    // Reduce certain Values
+                    AST::Value { token } => {
+                        log::trace!("Reducing value: {}", token);
+                        match token.ttype {
+                            // Don't reduce sequential identifiers or strings
+                            TokenType::Identifier | TokenType::String => {
+                                if let Some(next) = asts.get(i+1) {
+                                    match next {
+                                        AST::Macro { .. } => unchanged += 1,
+                                        AST::Value { token } => match token.ttype {
+                                            TokenType::Identifier | TokenType::Number | TokenType::String => unchanged += 1,
+                                            _ => unchanged = 0,
+                                        },
+                                        _ => unchanged = 0,
+                                    }
+                                } else {
+                                    unchanged = 0;
+                                }
                             },
-                        }
-                    },
-                    AST::Operator { token, lhs, rhs } if token.data != "=" && *lhs == AST::Empty && *rhs == AST::Empty => {
-                        // TODO operator precedence
-                        let lhs = if i > 0 {
-                            asts.remove(i-1)
-                        } else {
-                            i += 1;
-                            AST::Empty
-                        };
-                        if i > 0 {
-                            let _op = asts.remove(i-1); // index changed from previous remove
-                        } else {
-                            let _op = asts.remove(i);
-                            i += 1;
-                        }
-                        let rhs = if i > 0 && i <= asts.len() {
-                            asts.remove(i-1)
-                        } else {
-                            AST::Empty
-                        };
+                            // Combine number tokens
+                            TokenType::Number => {
+                                if let Some(next) = asts.get(i+1).cloned() {
+                                    match next {
+                                        AST::Value { token: t2 } => {
+                                            asts.remove(i+1);
+                                            asts.remove(i);
 
-                        unchanged = 0;
-                        asts.insert(i-1, AST::Operator {
-                            token,
-                            lhs: Box::new(lhs),
-                            rhs: Box::new(rhs),
-                        });
-                    },
-                    AST::Operator { token, lhs, rhs } if token.data == "=" && *lhs == AST::Empty && *rhs == AST::Empty => {
-                        let lhs = if i > 0 {
-                            asts.remove(i-1)
-                        } else {
-                            i += 1;
-                            AST::Empty
-                        };
-                        if i > 0 {
-                            let _op = asts.remove(i-1); // index changed from previous remove
-                        } else {
-                            let _op = asts.remove(i);
-                            i += 1;
+                                            asts.insert(i, AST::Value {
+                                                token: Token {
+                                                    ttype: TokenType::Number,
+                                                    pos: token.pos,
+                                                    data: token.data + &t2.data,
+                                                },
+                                            });
+
+                                            unchanged = 0;
+                                        },
+                                        AST::Container { token, .. } if token.data.starts_with("//") => unchanged = 0,
+                                        AST::Operator { .. } => unchanged = 0,
+                                        _ => unchanged += 1,
+                                    }
+                                }
+                            },
+                            _ => unchanged = 0,
                         }
-                        let rhs = if i > 0 && i <= asts.len() {
-                            asts.remove(i-1)
-                        } else {
-                            AST::Empty
-                        };
-
-                        unchanged = 0;
-                        asts.insert(i-1, AST::Operator {
-                            token,
-                            lhs: Box::new(lhs),
-                            rhs: Box::new(rhs),
-                        });
-                    },
-
-                    AST::Macro { .. } | AST::Container { .. } | AST::Value { .. } => {
-                        unchanged = 0;
                     },
                     _ => {
-                        // eprintln!("{:?}", asts[i]);
+                        log::trace!("{:?}", asts[i]);
                     },
                 }
+                // Error out if unable to reduce
                 if unchanged > 0 {
+                    if next_prec < Precedence::MAX as usize {
+                        i = 0;
+                        unchanged = 0;
+                        next_prec += 1;
+                        continue;
+                    }
+
                     return Err(Error::Parse(
                         format!("Unreducible values [{}/{}]: {:?}", i+1, asts.len(), asts.iter()
                             .map(|a| "<".to_string()+&a.to_string_lossy()+">")
@@ -380,7 +399,6 @@ impl AST {
                         asts[0].get_first_pos())
                     );
                 }
-                unchanged += 1;
                 i += 1;
             }
         }
@@ -395,12 +413,12 @@ impl AST {
     Runs the `AST` in the given environment and returns the value of the
     expression.
     */
-    pub fn run<'a>(&self, env: &'a Env<'a>) -> FnReturn {
+    pub fn run(&self, env: &Env<'_>) -> FnReturn {
         match self {
             AST::Macro { tokens } => {
                 let t0 = tokens.first().expect("expected macro token");
                 if let Some(rv) = env.get(&t0.data) {
-                    if let TVal { val: Value::Function { body, .. }, .. } = rv.clone_out() {
+                    if let Value::Function { body, .. } = rv.clone_out().val {
                         return body.call(&*env, FnArgs::Macro {
                             vars: Env::flatten(env).vars.into(),
                             pos: self.get_first_pos(),
@@ -411,7 +429,7 @@ impl AST {
                 return (None, Err(Error::Script(format!("failed to find macro {} in env", t0.data), self.get_first_pos())));
             },
             AST::Container { token, children } => {
-                let mut nenv = Env::child(&env);
+                let mut nenv = Env::child(env);
                 let mut should_map = false;
                 let vals: Result<Vec<RefTVal>, Error> = children.iter()
                     .filter_map(|ast| {
@@ -429,13 +447,13 @@ impl AST {
                         // Run child AST
                         let (vars, v) = ast.run(&nenv);
                         if let Some(vars) = vars {
+                            log::trace!("{:?}", vars.keys());
                             nenv.update(vars);
-                            // eprintln!("{:?}", vars);
                             // crate::run_interactive(&nenv);
                         }
                         Some(v)
                     }).collect();
-                let diff = env.diff(&mut nenv.clone());
+                let diff = env.diff(nenv.clone());
                 let vals: Vec<RefTVal> = match vals {
                     Ok(vs) => vs,
                     Err(m) => return (Some(diff), Err(m)),
@@ -455,7 +473,7 @@ impl AST {
                         if should_map {
                             let map: Result<Vec<(RefTVal, RefTVal)>, Error> = vals.iter()
                                 .map(|rv| {
-                                    if let TVal { val: Value::List(l), .. } = rv.clone_out() {
+                                    if let Value::List(l) = rv.clone_out().val {
                                         if l.len() == 2 {
                                             return Ok((l[0].clone(), l[1].clone()));
                                         }
@@ -477,6 +495,21 @@ impl AST {
                 let mut lval = None;
                 let mut rval = None;
 
+                // Handle the Dollar operator without evaluating either side
+                if token.data == "$" {
+                    let args = match &**lhs {
+                        AST::Value { token: t2 } => Arc::new(vec![
+                            (t2.data.clone(), Type::any()),
+                        ]),
+                        _ => return (None, Err(Error::Script(format!("invalid function parameter: {}", lhs.to_string_lossy()), Some(token.pos.clone())))),
+                    };
+                    return (None, Ok(Value::Function {
+                        args,
+                        vars: Env::flatten(env).vars.into(),
+                        body: Callable::AST(*rhs.clone()),
+                    }.into()));
+                }
+
                 // Handle the Equal operator without evaluating the LHS
                 if token.data == "=" {
                     let lval: RefTVal = match &**lhs {
@@ -499,7 +532,7 @@ impl AST {
                         Ok(v) => Some(v),
                         Err(m) => return (None, Err(m)),
                     };
-                    return OpTrait::<'a, {Op::Equal}>::op(&lval, &env, token.pos.clone(), rval.as_ref());
+                    return OpTrait::<{Op::Equal}>::op(&lval, env, token.pos.clone(), rval.as_ref());
                 }
 
                 // Compute lval
@@ -525,20 +558,25 @@ impl AST {
                             }
                         },
                         _ => {
-                            let rstr = rhs.get_first_token().expect("expected RHS token").data;
+                            let rstr = match rhs.get_first_token() {
+                                Some(t) => t.data,
+                                None => return (None, Err(Error::Script(format!("expected RHS token for operator {}", token.data), Some(token.pos.clone())))),
+                            };
                             (rstr.clone(), Some(Value::String(rstr).into()))
                         },
                     };
                     match lval {
-                        Some(lval) => return Op::op(&env, token.clone(), Some(&lval), rval.as_ref()),
+                        Some(lval) => return Op::op(env, token.clone(), Some(&lval), rval.as_ref()),
                         None if token.data == "." => {
                             // If no LHS then fetch from the Enums in the current env
-                            for (_pek, pev) in Env::iter_flatten(&env) {
-                                if let TVal { val: Value::Type(en, enr), .. } = pev.clone_out() {
-                                    if let Type::Enum(envar) = &*en {
-                                        if enr.get(&rstr).is_some() {
-                                            let v = Value::Enum(Arc::clone(envar), Box::new((rstr, vec![])));
-                                            return (None, Ok(v.into()));
+                            for (_pek, pev) in Env::iter_flatten(env) {
+                                if let Ok(pev) = pev.lock() {
+                                    if let Value::Type(en, enr) = &pev.val {
+                                        if let Type::Enum(envar) = &**en {
+                                            if enr.get(&rstr).is_some() {
+                                                let v = Value::Enum(Arc::clone(envar), Box::new((rstr, vec![])));
+                                                return (None, Ok(v.into()));
+                                            }
                                         }
                                     }
                                 }
@@ -558,11 +596,8 @@ impl AST {
                 }
 
                 // Compute operation
-                let (vars, val) = Op::op(&env, token.clone(), lval.as_ref(), rval.as_ref());
-                let nvars = match vars {
-                    Some(vars) => Some(env.diff(&mut Env::from(vars))),
-                    None => None,
-                };
+                let (vars, val) = Op::op(env, token.clone(), lval.as_ref(), rval.as_ref());
+                let nvars = vars.map(|vs| env.diff(Env::from(vs)));
                 return (nvars, val);
             },
             AST::Value { token } => {
@@ -571,42 +606,12 @@ impl AST {
                         Some(v) => return (None, Ok(v.clone())),
                         None => return (None, Err(Error::Script(format!("failed to find '{}' in this env", token.data), self.get_first_pos()))),
                     },
-                    TokenType::Number => {
-                        match BigRational::from_str(&token.data) {
-                            Ok(r) => return (None, Ok(Value::Number(r).into())),
-                            Err(_m) => {
-                                let (mut td, ordmag) = if let Some(eidx) = token.data.to_lowercase().find('e') {
-                                    (token.data[..eidx].to_string(), match BigInt::from_str(&token.data[eidx+1..]) {
-                                        Ok(v) => v,
-                                        Err(m) => return (None, Err(Error::Script(format!("failed to parse '{}' as number: {}", token.data, m), self.get_first_pos()))),
-                                    })
-                                } else {
-                                    (token.data.clone(), BigInt::zero())
-                                };
-
-                                let denom = if let Some(didx) = td.find('.') {
-                                    td.remove(didx);
-                                    BigInt::from_usize(td.len() - didx).unwrap_or_else(BigInt::one)
-                                } else {
-                                    BigInt::zero()
-                                };
-
-                                match (|| -> Option<BigRational> {
-                                    let ten = BigRational::from_i64(10)?;
-                                    let r = BigRational::from_str(&td).ok()?;
-                                    Some(r * ten.pow((ordmag -denom).to_i32()?))
-                                })() {
-                                    Some(r) => return (None, Ok(Value::Number(r).into())),
-                                    None => return (None, Err(Error::Script(format!("failed to parse '{}' as number", token.data), self.get_first_pos()))),
-                                }
-                            },
-                        }
-                    },
+                    TokenType::Number => return parse_number(&token.data, self.get_first_pos()),
                     TokenType::String => return (None, Ok(Value::String(unescape(&token.data[1..token.data.len()-1])).into())),
                     _ => {},
                 }
             },
-            _ => {},
+            AST::Empty => {},
         }
         (None, Err(Error::Script(format!("failed to execute AST: {:?}", self), self.get_first_pos())))
     }
