@@ -1,15 +1,11 @@
 use std::fmt;
 
-use num::ToPrimitive;
-
-#[cfg(feature = "compile")]
-use serde::{Serialize, Deserialize};
+use num_traits::ToPrimitive;
 
 use crate::Value;
 
 /// Denotes a specific position within a script.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct Pos {
     pub filename: String,
     pub line: usize,
@@ -124,7 +120,7 @@ impl<'a> SplitItem<'a> {
 trait SplitResult {
     fn new(pos: Pos, data: String) -> Self;
 }
-fn split<'a, SR>(si: &mut SplitItem<'a>, schar: char) -> Option<SR>
+fn split<SR>(si: &mut SplitItem<'_>, schar: char) -> Option<SR>
 where
     SR: SplitResult,
 {
@@ -170,7 +166,7 @@ where
                                     break;
                                 },
                                 None => break,
-                                _ => {},
+                                Some(_) => {},
                             }
                         }
                         break;
@@ -290,7 +286,6 @@ impl Iterator for Exprs {
 
 /// Easily differentiates `Token`s.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum TokenType {
     /// Anything alphanumeric not beginning with a number.
     Identifier,
@@ -333,7 +328,6 @@ A piece of the tokenized script.
 Usually a singular piece but may be a `Container`.
 */
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct Token {
     pub ttype: TokenType,
     pub pos: Pos,
@@ -362,13 +356,13 @@ impl fmt::Display for Token {
 #[derive(Debug, Clone)]
 pub struct Tokens {
     pos: Pos,
-    remainder: String,
+    remainder: Vec<char>,
 }
 impl Tokens {
     pub fn tokenize(expr: &Expr) -> Tokens {
         Tokens {
             pos: expr.pos.clone(),
-            remainder: expr.data.clone(),
+            remainder: expr.data.chars().collect(),
         }
     }
 }
@@ -379,7 +373,7 @@ impl Iterator for Tokens {
             return None;
         }
 
-        let remainder: Vec<char> = self.remainder.chars().collect();
+        let remainder: &Vec<char> = &self.remainder;
         let mut tc: usize = 0;
         // Eat whitespace before token
         while tc < remainder.len() && remainder[tc].is_whitespace() {
@@ -418,8 +412,13 @@ impl Iterator for Tokens {
         };
         tc += 1;
 
+        let mut containers = vec![];
+        if ttype == TokenType::Container {
+            containers.push(remainder[tc-1]);
+        }
+
         // Find the next token change
-        while tc < remainder.len() {
+        'outer: while tc < remainder.len() {
             let nc = remainder[tc];
             match nc {
                 '\n' => {
@@ -441,30 +440,24 @@ impl Iterator for Tokens {
                     // Allow numbers, a single period, and a single 'e'/'E'
                     // TODO add support for hex and binary numbers
                     if !nc.is_numeric() {
-                        let token = self.remainder.get(0..=tc)?.trim();
-                        if token.matches('.').count() > 1
-                            || token.to_lowercase().matches('e').count() > 1
-                            || !token.to_lowercase()
-                                .replace('.', "")
-                                .replace('e', "")
-                                .chars().all(char::is_numeric)
-                        {
-                            // Check that if there's a dot that it's not for Op::Dot
-                            let dot = token.find('.');
-                            if let Some(dot) = dot {
-                                if let Some(dec) =  token.get(dot+1..) {
-                                    if !dec.chars().all(char::is_numeric) {
-                                        tc -= 1;
+                        let token = slice_trim(self.remainder.get(0..=tc)?);
+                        for (i, c) in token.iter().enumerate() {
+                            if !c.is_numeric() && c != &'.' && c != &'e' && c != &'E' {
+                                if c == &'.' {
+                                    if let Some(dec) = token.get(i+1..) {
+                                        if !dec.iter().all(|c| c.is_numeric()) {
+                                            tc -= 1;
+                                        }
                                     }
                                 }
+                                break 'outer;
                             }
-                            break;
                         }
                     }
                 },
                 TokenType::String => {
                     // Remove escaped backslashes then remove escaped quotes to determine actual quotes
-                    let token = self.remainder.get(0..=tc)?.trim();
+                    let token = slice_trim(self.remainder.get(0..=tc)?).iter().collect::<String>();
                     let unescaped = token.replace(r"\\", "")
                         .replace(r#"\""#, "");
 
@@ -475,32 +468,30 @@ impl Iterator for Tokens {
                 },
                 TokenType::Container => {
                     // Check container matching
-                    let mut containers = vec![];
-                    let token: Vec<char> = self.remainder.get(0..=tc)?.trim()
-                        .chars().collect();
-                    for c in &token {
-                        match c {
-                            '(' | '{' | '[' => containers.push(c),
-                            ')' => {
-                                if containers.is_empty() || containers.last().unwrap() != &&'(' {
-                                    panic!("mismatched containers at {}: {:?}", self.pos, token);
-                                }
-                                containers.pop();
-                            },
-                            '}' => {
-                                if containers.is_empty() || containers.last().unwrap() != &&'{' {
-                                    panic!("mismatched containers at {}: {:?}", self.pos, token);
-                                }
-                                containers.pop();
-                            },
-                            ']' => {
-                                if containers.is_empty() || containers.last().unwrap() != &&'[' {
-                                    panic!("mismatched containers at {}: {:?}", self.pos, token);
-                                }
-                                containers.pop();
-                            },
-                            _ => {},
-                        }
+                    match nc {
+                        '(' | '{' | '[' => containers.push(nc),
+                        ')' => {
+                            if containers.is_empty() || containers.last().unwrap() != &'(' {
+                                let token = slice_trim(self.remainder.get(0..=tc)?);
+                                panic!("mismatched containers at {}: {:?}", self.pos, token);
+                            }
+                            containers.pop();
+                        },
+                        '}' => {
+                            if containers.is_empty() || containers.last().unwrap() != &'{' {
+                                let token = slice_trim(self.remainder.get(0..=tc)?);
+                                panic!("mismatched containers at {}: {:?}", self.pos, token);
+                            }
+                            containers.pop();
+                        },
+                        ']' => {
+                            if containers.is_empty() || containers.last().unwrap() != &'[' {
+                                let token = slice_trim(self.remainder.get(0..=tc)?);
+                                panic!("mismatched containers at {}: {:?}", self.pos, token);
+                            }
+                            containers.pop();
+                        },
+                        _ => {},
                     }
 
                     // If the original container is closed then we're done
@@ -510,25 +501,25 @@ impl Iterator for Tokens {
                     }
                 },
                 TokenType::Symbol => {
-                    let token = self.remainder.get(0..=tc)?.trim();
-                    match token {
-                        "::" |
-                        "**" |
-                        "<<" |
-                        ">>" |
-                        "<=>" |
-                        "<=" |
-                        ">=" |
-                        "==" |
-                        "!=" |
-                        "&&" |
-                        "||" |
-                        ".." |
-                        "..=" |
-                        "->" |
-                        "<-" |
-                        "=>" |
-                        "|>" => {},
+                    let token = slice_trim(self.remainder.get(0..=tc)?);
+                    match *token {
+                        [':', ':'] |
+                        ['*', '*'] |
+                        ['<', '<'] |
+                        ['>', '>'] |
+                        ['<', '=', '>'] |
+                        ['<', '='] |
+                        ['>', '='] |
+                        ['=', '='] |
+                        ['!', '='] |
+                        ['&', '&'] |
+                        ['|', '|'] |
+                        ['.', '.'] |
+                        ['.', '.', '='] |
+                        ['-', '>'] |
+                        ['<', '-'] |
+                        ['=', '>'] |
+                        ['|', '>'] => {},
                         _ => break,
                     }
                 },
@@ -538,10 +529,14 @@ impl Iterator for Tokens {
         }
 
         // Split off the front up to the found token change
-        let token = self.remainder.get(0..tc)?.trim().to_string();
+        let token = slice_trim(self.remainder.get(0..tc)?).iter().collect::<String>();
+        if !containers.is_empty() {
+            panic!("mismatched containers at {}: {:?}", self.pos, token);
+        }
+
         self.remainder = self.remainder.get(tc..)
-            .unwrap_or("")
-            .to_string();
+            .unwrap_or(&[])
+            .to_vec();
         if token.is_empty() {
             return None;
         }
@@ -555,7 +550,7 @@ impl Iterator for Tokens {
 
 pub fn unescape(s: &str) -> String {
     s.chars()
-        .fold((String::new(), None), |(mut ns, prev), c| {
+        .fold((String::with_capacity(s.len()), None), |(mut ns, prev), c| {
             if let Some('\\') = prev {
                 match c {
                     '\\' => {
@@ -583,3 +578,11 @@ pub fn unescape(s: &str) -> String {
             (ns, Some(c))
         }).0
 }
+pub fn slice_trim(v: &[char]) -> &[char] {
+    let from = match v.iter().position(|c| !c.is_whitespace()) {
+        Some(i) => i,
+        None => return &[],
+    };
+    let to = v.iter().rposition(|c| !c.is_whitespace()).unwrap();
+    &v[from..=to]
+}
