@@ -2,17 +2,17 @@ use std::fmt;
 use std::collections::HashMap;
 use std::sync::Arc;
 use std::convert::TryFrom;
+use std::str::FromStr;
 
-use num::{BigRational, BigInt, One, ToPrimitive, FromPrimitive};
+use num_rational::BigRational;
+use num_bigint::BigInt;
+use num_traits::{One, Zero, ToPrimitive, FromPrimitive};
 
 use maplit::hashmap;
 
-#[cfg(feature = "compile")]
-use serde::{Serialize, Deserialize};
-
 use lazy_static::lazy_static;
 
-use crate::{HashableMap, Error, Env, Pos, TokenType, Token, AST, RefTVal, ThreadHandle};
+use crate::{HashableMap, InternalData, Error, Env, Pos, TokenType, Token, AST, RefTVal, ThreadHandle};
 
 /// Function arguments for various types of functions.
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -55,7 +55,7 @@ impl Callable {
     Returns the value of the function call in the given environment with the
     given `FnArgs`.
     */
-    pub fn call<'a>(&self, env: &'a Env<'a>, args: FnArgs) -> FnReturn {
+    pub fn call(&self, env: &Env<'_>, args: FnArgs) -> FnReturn {
         match self {
             Callable::Native(fp) => fp(args),
             Callable::Value(rv) => (None, Ok(rv.clone())),
@@ -65,10 +65,10 @@ impl Callable {
                 if let FnArgs::Macro { vars, pos, tokens } = &args {
                     nenv.set("env", &Value::from_env(&Env::from(vars.clone())).into());
                     nenv.set("pos", &match pos {
-                        Some(p) => Value::from_pos(&p).into(),
+                        Some(p) => Value::from_pos(p).into(),
                         None => Value::none().into(),
                     });
-                    nenv.set("tokens", &Value::from_tokens(&tokens).into());
+                    nenv.set("tokens", &Value::from_tokens(tokens).into());
                 }
 
                 // Run function AST
@@ -86,17 +86,29 @@ impl Callable {
                     FnArgs::Macro { pos, .. } => {
                         match &val {
                             Ok(rv) => {
-                                if let TVal { val: Value::List(l), .. } = rv.clone_out() {
+                                if let Value::List(l) = rv.clone_out().val {
                                     // Get vars
                                     if let Some(rv0) = l.get(0) {
-                                        if let TVal { val: Value::Enum(_, e1), .. } = rv0.clone_out() {
+                                        if let Value::Enum(_, e1) = rv0.clone_out().val {
                                             if &*e1.0 == "Some" {
                                                 let hm = match e1.1.get(0) {
                                                     Some(rv) => {
-                                                        if let Value::Struct(hm) = rv.clone_out().val {
-                                                            hm
-                                                        } else {
-                                                            return (None, Err(Error::Script(format!("expected Struct for new env: {}", rv), pos)))
+                                                        match rv.clone_out().val {
+                                                            Value::Struct(hm) => hm,
+                                                            Value::Map(m) => {
+                                                                let hm: Result<HashableMap<String, RefTVal>, Error> = m.iter()
+                                                                    .map(|(k, v)| {
+                                                                        match k.clone_out().val {
+                                                                            Value::String(s) => Ok((s, v.clone())),
+                                                                            _ => Err(Error::Script(format!("expected String key in env Map, got {}", k), pos.clone())),
+                                                                        }
+                                                                    }).collect();
+                                                                match hm {
+                                                                    Ok(hm) => hm,
+                                                                    Err(m) => return (None, Err(m)),
+                                                                }
+                                                            },
+                                                            _ => return (None, Err(Error::Script(format!("expected Struct or Map for new env: {}", rv), pos))),
                                                         }
                                                     },
                                                     None => return (None, Err(Error::Script(format!("expected new env: {:?}", e1), pos))),
@@ -111,7 +123,7 @@ impl Callable {
 
                                     // Get val
                                     if let Some(rv1) = l.get(1) {
-                                        if let TVal { val: Value::Enum(_, e2), .. } = rv1.clone_out() {
+                                        if let Value::Enum(_, e2) = rv1.clone_out().val {
                                             let e2v = e2.1.get(0).cloned().unwrap_or_else(|| Value::none().into());
                                             match &*e2.0 {
                                                 "Ok" => {
@@ -134,9 +146,7 @@ impl Callable {
                                                         None
                                                     };
                                                     if let Some(e) = e {
-                                                        // eprintln!("{:?} {:?}", l2, e.0);
                                                         let err = Value::Enum(Arc::new(vec![]), e).to_error();
-                                                        // eprintln!("{:?}", err);
                                                         if let Some(err) = err {
                                                             return (vars, Err(err));
                                                         }
@@ -146,7 +156,6 @@ impl Callable {
                                                 _ => {},
                                             }
                                         }
-
                                         return (vars, val);
                                     }
                                 }
@@ -155,7 +164,7 @@ impl Callable {
                         }
                         return (None, Err(Error::Script(format!("failed to get FnReturn from user macro {:?}", val), None)));
                     },
-                    _ => (None, val),
+                    FnArgs::Normal { .. } => (None, val),
                 }
             },
         }
@@ -169,7 +178,6 @@ corresponding `Value` variants.
 The below associated functions return cloned `Arc`s to defaults.
 */
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum Type {
     /**
     Null type, only to be used internally. For external code, use an `Option`.
@@ -182,7 +190,7 @@ pub enum Type {
     Any,
 
     Type,
-    Reference,
+    Reference(Arc<Type>),
     Number,
     String,
 
@@ -210,7 +218,7 @@ lazy_static! {
     pub(self) static ref TYPE_NONE:   Arc<Type> = Arc::new(Type::None);
     pub(self) static ref TYPE_ANY:    Arc<Type> = Arc::new(Type::Any);
     pub(self) static ref TYPE_TYPE:   Arc<Type> = Arc::new(Type::Type);
-    pub(self) static ref TYPE_REF:    Arc<Type> = Arc::new(Type::Reference);
+    pub(self) static ref TYPE_REF:    Arc<Type> = Arc::new(Type::Reference(Type::none()));
     pub(self) static ref TYPE_NUMBER: Arc<Type> = Arc::new(Type::Number);
     pub(self) static ref TYPE_STRING: Arc<Type> = Arc::new(Type::String);
 
@@ -295,7 +303,10 @@ impl Type {
             Type::None => matches!(other, Type::None),
             Type::Any => matches!(other, Type::Any),
             Type::Type => matches!(other, Type::Type),
-            Type::Reference => matches!(other, Type::Reference),
+            Type::Reference(rt1) => match other {
+                Type::Reference(rt2) => rt1.eq(rt2),
+                _ => false,
+            },
             Type::Number => matches!(other, Type::Number),
             Type::String => matches!(other, Type::String),
 
@@ -316,7 +327,7 @@ impl fmt::Display for Type {
             Type::None => write!(f, "None"),
             Type::Any => write!(f, "Any"),
             Type::Type => write!(f, "Type"),
-            Type::Reference => write!(f, "Reference"),
+            Type::Reference(rt) => write!(f, "Reference[{}]", rt),
             Type::Number => write!(f, "Number"),
             Type::String => write!(f, "String"),
 
@@ -334,6 +345,10 @@ impl fmt::Display for Type {
 impl TryFrom<&str> for Type {
     type Error = Error;
     fn try_from(s: &str) -> Result<Self, Error> {
+        if s.starts_with('&') {
+            return Ok(Type::Reference(Arc::new(Type::try_from(s.strip_prefix('&').unwrap())?)));
+        }
+
         Ok(match s {
             "None" => Type::None,
             "Any" => Type::Any,
@@ -356,12 +371,14 @@ impl TryFrom<&str> for Type {
 
 /// A representation of any valid value in a script.
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub enum Value {
     /// A `Type` with subtype values.
     Type(Arc<Type>, Arc<HashableMap<String, RefTVal>>),
     /// A reference to another value.
     Reference(RefTVal),
+    /// Internal data for storing state
+    Internal(InternalData),
+
     /// Any rational number.
     Number(BigRational),
     /// Any UTF8 string.
@@ -402,20 +419,31 @@ pub enum Value {
     Thread(ThreadHandle),
 }
 lazy_static! {
-    static ref ENUM_TOKEN_TYPE: Arc<Vec<(String, Arc<Type>)>> = Arc::new(vec![
+    pub static ref ENUM_TOKEN_TYPE: Arc<Vec<(String, Arc<Type>)>> = Arc::new(vec![
         ("Identifier".into(), Type::none()),
         ("Number".into(), Type::none()),
         ("String".into(), Type::none()),
         ("Container".into(), Type::none()),
         ("Symbol".into(), Type::none()),
     ]);
-    static ref ENUM_AST: Arc<Vec<(String, Arc<Type>)>> = Arc::new(vec![
+    pub static ref ENUM_AST: Arc<Vec<(String, Arc<Type>)>> = Arc::new(vec![
         ("Macro".into(), Type::none()),
         ("Container".into(), Type::none()),
         ("Operator".into(), Type::none()),
         ("Value".into(), Type::none()),
         ("Empty".into(), Type::none()),
     ]);
+    pub static ref ENUM_VALUE: Arc<Vec<(String, Arc<Type>)>> = Arc::new(vec![
+        ("Type".into(), Type::none()),
+        ("Reference".into(), Type::none()),
+        ("Number".into(), Type::none()),
+        ("String".into(), Type::none()),
+        ("List".into(), Type::none()),
+        ("Map".into(), Type::none()),
+        ("Enum".into(), Type::none()),
+        ("Struct".into(), Type::none()),
+        ("Thread".into(), Type::none()),
+    ]);
 }
 impl Value {
     pub(crate) fn none() -> Self {
@@ -428,8 +456,10 @@ impl Value {
     */
     pub(crate) fn deep_clone(&self, attr: bool) -> Value {
         match self {
-            Value::Type(t, ts) => Value::Type(Arc::clone(&t), Arc::new(ts.deep_clone(attr))),
+            Value::Type(t, ts) => Value::Type(Arc::clone(t), Arc::new(ts.deep_clone(attr))),
             Value::Reference(rv) => Value::Reference(rv.deep_clone(attr)),
+            Value::Internal(id) => Value::Internal(id.clone()),
+
             Value::Number(_) | Value::String(_) => self.clone(),
 
             Value::List(l) => Value::List(l.iter().map(|v| v.deep_clone(attr)).collect()),
@@ -473,12 +503,12 @@ impl Value {
     }
     pub(crate) fn from_tokens(tokens: &[Token]) -> Self {
         tokens.iter()
-            .map(|t| Value::from_token(t))
+            .map(Value::from_token)
             .collect::<Vec<Value>>()
             .into()
     }
 
-    pub(crate) fn from_env<'a>(env: &Env<'a>) -> Self {
+    pub(crate) fn from_env(env: &Env<'_>) -> Self {
         let p: HashableMap<RefTVal, RefTVal> = match &env.parent {
             Some(p) => Env::flatten(p).vars.iter()
                 .map(|(k, v)| (Value::String(k.clone()).into(), v.clone()))
@@ -536,7 +566,7 @@ impl Value {
         match r {
             Ok(v) => Value::Enum(es, Box::new(("Ok".into(), vec![v.clone()]))),
             Err(m) => Value::Enum(es,
-                Box::new(("Err".into(), vec![Value::from_error(&m).into()]),
+                Box::new(("Err".into(), vec![Value::from_error(m).into()]),
             )),
         }
     }
@@ -563,11 +593,11 @@ impl Value {
                 Error::Argument(m) => ("ArgumentError".into(), vec![Value::String(m.clone()).into()]),
                 Error::Parse(m, p) => ("ParseError".into(), vec![
                     Value::String(m.clone()).into(),
-                    Value::from_option(&p.as_ref().map(|p| Value::from_pos(&p).into())).into(),
+                    Value::from_option(&p.as_ref().map(|p| Value::from_pos(p).into())).into(),
                 ]),
                 Error::Script(m, p) => ("ScriptError".into(), vec![
                     Value::String(m.clone()).into(),
-                    Value::from_option(&p.as_ref().map(|p| Value::from_pos(&p).into())).into(),
+                    Value::from_option(&p.as_ref().map(|p| Value::from_pos(p).into())).into(),
                 ]),
                 Error::Control(m, v) => ("ControlError".into(), vec![
                     Value::String(m.clone()).into(),
@@ -580,10 +610,7 @@ impl Value {
     pub(crate) fn to_error(&self) -> Option<Error> {
         if let Value::Enum(_, e) = self {
             match &*e.0 {
-                "ArgumentError" => match &e.1.get(0) {
-                    Some(rv) => Some(Error::Argument(rv.clone_out().val.to_string())),
-                    _ => None,
-                },
+                "ArgumentError" => e.1.get(0).map(|rv| Error::Argument(rv.clone_out().val.to_string())),
                 "ParseError" => match &e.1.get(1) {
                     Some(rv1) => match rv1.clone_out().val {
                         Value::Enum(_, e2) => {
@@ -624,14 +651,10 @@ impl Value {
                     },
                     _ => None,
                 },
-                "ControlError" => match &e.1.get(0) {
-                    Some(rv) => Some(Error::Control(rv.clone_out().val.to_string(), e.1.get(1).cloned().map(|v| Box::new(v.clone_out())))),
-                    _ => None,
-                },
-                "CustomError" => match &e.1.get(0) {
-                    Some(rv) => Some(Error::Custom(rv.clone_out())),
-                    _ => None,
-                },
+                "ControlError" => e.1.get(0)
+                    .map(|rv| Error::Control(rv.clone_out().val.to_string(), e.1.get(1).cloned().map(|v| Box::new(v.clone_out())))),
+                "CustomError" => e.1.get(0)
+                    .map(|rv| Error::Custom(rv.clone_out())),
                 _ => None,
             }
         } else {
@@ -668,6 +691,10 @@ impl fmt::Debug for Value {
             Value::Reference(rv) => {
                 write!(f, "Ref[{:?}]", rv)
             },
+            Value::Internal(p) => {
+                write!(f, "Internal[{:?}]", p)
+            },
+
             Value::Number(r) => {
                 if r.denom() == &BigInt::one() {
                     write!(f, "{}", r.numer())
@@ -676,7 +703,7 @@ impl fmt::Debug for Value {
                 }
             },
             Value::String(s) => {
-                write!(f, "{}", s)
+                write!(f, "{}", s) // FIXME make quoting for Debug and non-quoting for Display work
             },
 
             Value::List(l) => write!(f, "{:?}", {
@@ -721,7 +748,6 @@ impl fmt::Display for Value {
 
 /// A `Type`-`Value` pair along with an attribute map.
 #[derive(Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-#[cfg_attr(feature = "compile", derive(Serialize, Deserialize))]
 pub struct TVal {
     pub ttype: Arc<Type>,
     pub attr: HashableMap<String, RefTVal>,
@@ -760,11 +786,20 @@ impl From<Value> for TVal {
                 attr: hashmap!{}.into(),
                 val,
             },
-            V::Reference(_) => TVal {
-                ttype: T::reference(),
+            V::Reference(rv) => TVal {
+                ttype: Arc::new(T::Reference(match &rv.try_lock() {
+                    Ok(mutex) => mutex.ttype.clone(),
+                    Err(_) => T::none(),
+                })),
+                attr: hashmap!{}.into(),
+                val: val.clone(),
+            },
+            V::Internal(_) => TVal {
+                ttype: T::none(),
                 attr: hashmap!{}.into(),
                 val,
             },
+
             V::Number(_) => TVal {
                 ttype: T::number(),
                 attr: hashmap!{}.into(),
@@ -778,11 +813,7 @@ impl From<Value> for TVal {
 
             V::List(l) => TVal {
                 ttype: Arc::new(T::List({
-                    if let Some(a) = l.first() {
-                        Arc::clone(&a.clone_out().ttype)
-                    } else {
-                        T::none()
-                    }
+                    l.first().map_or_else(T::none, |a| Arc::clone(&a.clone_out().ttype))
                 })),
                 attr: hashmap!{}.into(),
                 val,
@@ -797,7 +828,7 @@ impl From<Value> for TVal {
                 val,
             },
             V::Enum(es, ..) => TVal {
-                ttype: Arc::new(T::Enum(Arc::clone(&es))),
+                ttype: Arc::new(T::Enum(Arc::clone(es))),
                 attr: hashmap!{}.into(),
                 val,
             },
@@ -815,7 +846,7 @@ impl From<Value> for TVal {
 
             V::Function { args, .. } => TVal {
                 ttype: Arc::new(T::Function {
-                    args: Arc::clone(&args),
+                    args: Arc::clone(args),
                     ret: T::none(), // TODO find return type
                 }),
                 attr: hashmap!{}.into(),
@@ -933,3 +964,59 @@ pub fn match_pat(tv: &RefTVal, pat: &AST, pos: Option<Pos>) -> Result<Option<Has
         }).data
     ), pos))
 }
+
+pub fn parse_number(s: &str, pos: Option<Pos>) -> FnReturn {
+    // Attempt straightforward parsing
+    match BigRational::from_str(s) {
+        Ok(r) => (None, Ok(Value::Number(r).into())),
+        Err(_m) => {
+            // Check for hex
+            if s.to_lowercase().find('x').is_some() {
+                // Attempt to parse hex
+                if let Ok(h) = u128::from_str_radix(&s[2..], 16) {
+                    if let Some(r) = BigRational::from_u128(h) {
+                        return (None, Ok(Value::Number(r).into()));
+                    }
+                }
+                return (None, Err(Error::Script(format!("failed to parse hex '{}' as number", s), pos)));
+            }
+
+            // Check for binary
+            if s.to_lowercase().find('b').is_some() {
+                // Attempt to parse hex
+                if let Ok(h) = u128::from_str_radix(&s[2..], 2) {
+                    if let Some(r) = BigRational::from_u128(h) {
+                        return (None, Ok(Value::Number(r).into()));
+                    }
+                }
+                return (None, Err(Error::Script(format!("failed to parse binary '{}' as number", s), pos)));
+            }
+
+            // Check for e/E exponent
+            let (mut td, ordmag) = if let Some(eidx) = s.to_lowercase().find('e') {
+                (s[..eidx].to_string(), match BigInt::from_str(&s[eidx+1..]) {
+                    Ok(v) => v,
+                    Err(m) => return (None, Err(Error::Script(format!("failed to parse '{}' as number: {}", s, m), pos))),
+                })
+            } else {
+                (s.to_string(), BigInt::zero())
+            };
+
+            // Check for decimal
+            let denom = td.find('.').map_or_else(BigInt::zero, |didx| {
+                td.remove(didx);
+                BigInt::from_usize(td.len() - didx).unwrap_or_else(BigInt::one)
+            });
+
+            // Attempt parsing decimal with exponent
+            match (|| -> Option<BigRational> {
+                let ten = BigRational::from_i64(10)?;
+                let r = BigRational::from_str(&td).ok()?;
+                Some(r * ten.pow((ordmag -denom).to_i32()?))
+            })() {
+                Some(r) => (None, Ok(Value::Number(r).into())),
+                None => (None, Err(Error::Script(format!("failed to parse '{}' as number", s), pos))),
+            }
+        },
+    }
+}
