@@ -2,12 +2,15 @@ use std::path::Path;
 use std::collections::HashMap;
 
 use maplit::hashmap;
+use num_rational::BigRational;
+use num_traits::One;
 
-use crate::{Error, Env, Value, TVal, RefTVal, TokenType, FnArgs, FnReturn, AST, run_path, unescape, ummod};
+use crate::hashablemap::HashableMap;
+use crate::{Error, Env, Value, RefTVal, TokenType, FnArgs, FnReturn, AST, run_path, unescape, ummod};
 
 use super::_parse_margs;
 
-// Base Macros: use, let
+// Base Macros: use, let, mod
 
 pub fn r#use(args: FnArgs) -> FnReturn {
     let (env, pos, tokens) = match _parse_margs("macro use", args) {
@@ -24,7 +27,7 @@ pub fn r#use(args: FnArgs) -> FnReturn {
     };
     let mp = match &modpath {
         Some(rv) => {
-            if let TVal { val: Value::String(mn), .. } = rv.clone_out() {
+            if let Value::String(mn) = rv.clone_out().val {
                 mn
             } else {
                 return (None, Err(Error::Script(format!("macro use: invalid module path {}", m), pos)))
@@ -57,21 +60,20 @@ pub fn r#use(args: FnArgs) -> FnReturn {
         .join(modpath);
 
     // Import module
-    let menv = if env.has("__prelude") {
-        Env::core()
-    } else {
-        Env::prelude()
-    };
-    let (vars, val): FnReturn = if let Some(vars) = ummod::get(&modname) {
-        (Some(vars), Ok(Value::none().into()))
-    } else {
-        let mpath = Path::new("mod").join(modname.clone());
-        match run_path(&mpath, &menv, false) {
-            (vars, Ok(val)) => (vars, Ok(val)),
-            (_, Err(Error::Argument(_))) => run_path(&modpath, &menv, false),
-            (vars, Err(m)) => (vars, Err(m)),
-        }
-    };
+    let (vars, val): FnReturn = ummod::get(&modname).map_or_else(
+        || {
+            let menv = if env.has("__prelude") {
+                Env::core()
+            } else {
+                Env::prelude()
+            };
+            let mpath = Path::new("mod").join(modname.clone());
+            match run_path(&mpath, &menv, false) {
+                (vars, Ok(val)) => (vars, Ok(val)),
+                (_, Err(Error::Argument(_))) => run_path(&modpath, &menv, false),
+                (vars, Err(m)) => (vars, Err(m)),
+            }
+        }, |vars| (Some(vars), Ok(Value::none().into())));
     if let Err(m) = val {
         return (None, Err(m));
     }
@@ -90,11 +92,26 @@ pub fn r#use(args: FnArgs) -> FnReturn {
         for a in modas {
             match vars.get(&a) {
                 Some(v) => {
-                    hm.insert(a, v.clone());
+                    let mut tv = v.clone_out();
+                    if tv.attr.contains_key("pub") {
+                        tv.attr.remove("pub");
+                    }
+                    hm.insert(a, tv.into());
                 },
                 None => {
                     if a == "self" {
                         hm.insert(modname.clone(), modst.clone());
+                    } else if a == "*" {
+                        // Import all public items
+                        for (k, v) in vars.clone() {
+                            let mut tv = v.clone_out();
+                            if let Some(p) = tv.attr.get("pub") {
+                                if p == &Value::Number(BigRational::one()).into() {
+                                    tv.attr.remove("pub");
+                                    hm.insert(k, tv.into());
+                                }
+                            }
+                        }
                     } else {
                         return (None, Err(Error::Script(format!("macro use: failed to find name {} in module {}", a, modname), pos)))
                     }
@@ -118,9 +135,29 @@ pub fn r#let(args: FnArgs) -> FnReturn {
     let (_, val) = ast.run(&env);
 
     match val {
-        Ok(v) => match v.clone_out().val {
-            Value::Reference(rv) => (Some(hashmap!{ varname.clone() => rv.clone() }), Ok(rv)),
-            _ => (Some(hashmap!{ varname.clone() => v.clone() }), Ok(v)),
+        Ok(v) => (Some(hashmap!{ varname.clone() => v.clone() }), Ok(v)),
+        Err(m) => (None, Err(m)),
+    }
+}
+pub fn r#mod(args: FnArgs) -> FnReturn {
+    let (env, _pos, tokens) = match _parse_margs("macro mod", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    let ast = match AST::parse(tokens[1..].iter().cloned(), &env) {
+        Ok(a) => a,
+        Err(m) => return (None, Err(m)),
+    };
+    let (nenv, val) = ast.run(&env);
+
+    match val {
+        Ok(_) => {
+            let hm: HashableMap<String, RefTVal> = match nenv {
+                Some(hm) => hm.into(),
+                None => hashmap!{}.into(),
+            };
+            (None, Ok(Value::Struct(hm).into()))
         },
         Err(m) => (None, Err(m)),
     }
