@@ -3,7 +3,7 @@ use std::sync::Arc;
 
 use maplit::hashmap;
 
-use crate::{HashableMap, Error, Env, Type, Value, TVal, RefTVal, Token, FnArgs, FnReturn, Callable, AST};
+use crate::{HashableMap, Error, Env, Type, Value, RefTVal, Token, FnArgs, FnReturn, Callable, AST};
 
 use super::_parse_fargs;
 
@@ -37,17 +37,17 @@ pub fn env_child(fnargs: FnArgs) -> FnReturn {
 
     if let Value::List(l) = &args.clone_out().val {
         if let Some(rv0) = l.get(0) {
-            if let TVal { val: Value::Struct(penv), .. } = rv0.clone_out() {
+            if let Value::Struct(penv) = rv0.clone_out().val {
                 let pvars = match penv.get("vars") {
-                    Some(vs) => match vs.clone_out() {
-                        TVal { val: Value::Map(vs), .. } => vs,
+                    Some(vs) => match vs.clone_out().val {
+                        Value::Map(vs) => vs,
                         _ => return (None, Err(Error::Script(format!("function Env.child: env.vars is not a map {}", vs), pos))),
                     },
                     None => return (None, Err(Error::Script(format!("function Env.child: env doesn't contain vars {}", penv), pos))),
                 };
                 let vars: Result<HashMap<std::string::String, RefTVal>, Error> = pvars.iter()
-                    .map(|(k, v)| Ok((match k.clone_out() {
-                        TVal { val: Value::String(s), .. } => s,
+                    .map(|(k, v)| Ok((match k.clone_out().val {
+                        Value::String(s) => s,
                         _ => return Err(Error::Script(format!("function Env.child: expected Env[String, Any], got {}", penv), pos.clone())),
                     }, v.clone())))
                     .collect();
@@ -60,9 +60,9 @@ pub fn env_child(fnargs: FnArgs) -> FnReturn {
 
                 // Optionally set vars
                 if let Some(rv1) = l.get(1) {
-                    if let TVal { val: Value::Map(hm), .. } = rv1.clone_out() {
+                    if let Value::Map(hm) = rv1.clone_out().val {
                         for (k, v) in hm.iter() {
-                            if let TVal { val: Value::String(k), ..} = k.clone_out() {
+                            if let Value::String(k) = k.clone_out().val {
                                 ntenv.set(&k, &v.clone());
                             }
                         }
@@ -84,10 +84,10 @@ pub fn env_update(fnargs: FnArgs) -> FnReturn {
 
     if let Value::List(l) = &args.clone_out().val {
         if let Some(rv0) = l.get(0) {
-            if let TVal { val: Value::Struct(ref mut senv), .. } = &mut *rv0.try_lock().unwrap() {
+            if let Value::Struct(ref mut senv) = &mut rv0.try_lock().unwrap().val {
                 let mut svars = match senv.get("vars") {
-                    Some(vs) => match vs.clone_out() {
-                        TVal { val: Value::Map(vs), .. } => vs,
+                    Some(vs) => match vs.clone_out().val {
+                        Value::Map(vs) => vs,
                         _ => return (None, Err(Error::Script(format!("function Env.update: self.vars is not a map {}", vs), pos))),
                     },
                     None => return (None, Err(Error::Script(format!("function Env.update: self env doesn't contain vars {}", senv), pos))),
@@ -147,9 +147,9 @@ pub fn ast_parse(fnargs: FnArgs) -> FnReturn {
 
     if let Value::List(l) = &args.clone_out().val {
         if let Some(rv0) = l.get(0) {
-            if let TVal { val: Value::List(tokens), .. } = rv0.clone_out() {
+            if let Value::List(tokens) = rv0.clone_out().val {
                 if let Some(rv1) = l.get(1) {
-                    if let TVal { val: Value::Struct(env), .. } = rv1.clone_out() {
+                    if let Value::Struct(env) = rv1.clone_out().val {
                         let tokens: Result<Vec<Token>, Error> = tokens.iter()
                             .map(|t| Token::from_value(&t.clone_out().val).ok_or_else(|| Error::Script(format!("function AST.parse: expected token {}", t), pos.clone())))
                             .collect();
@@ -159,8 +159,8 @@ pub fn ast_parse(fnargs: FnArgs) -> FnReturn {
                         };
 
                         let vars = match env.get("vars") {
-                            Some(vs) => match vs.clone_out() {
-                                TVal { val: Value::Map(vs), .. } => vs,
+                            Some(vs) => match vs.clone_out().val {
+                                Value::Map(vs) => vs,
                                 _ => return (None, Err(Error::Script(format!("function AST.parse: env.vars is not a map {}", vs), pos))),
                             },
                             None => return (None, Err(Error::Script(format!("function AST.parse: env doesn't contain vars {}", env), pos))),
@@ -196,24 +196,24 @@ pub fn ast_run(fnargs: FnArgs) -> FnReturn {
 
     if let Value::List(l) = &args.clone_out().val {
         if let Some(rv0) = l.get(0) {
-            let TVal { val: ast, .. } = rv0.clone_out();
+            let ast = rv0.clone_out().val;
             if let Some(rv1) = l.get(1) {
-                if let TVal { val: Value::Struct(env), .. } = rv1.clone_out() {
+                if let Value::Struct(env) = rv1.clone_out().val {
                     let ast = match AST::from_value(&ast) {
                         Some(a) => a,
                         None => return (None, Err(Error::Script(format!("function AST.run: expected AST {}", ast), pos))),
                     };
 
                     let vars: HashableMap<RefTVal, RefTVal> = match env.get("vars") {
-                        Some(vs) => match vs.clone_out() {
-                            TVal { val: Value::Map(vs), .. } => vs,
+                        Some(vs) => match vs.clone_out().val {
+                            Value::Map(vs) => vs,
                             _ => return (None, Err(Error::Script(format!("function AST.run: env.vars is not a map {}", vs), pos))),
                         },
                         None => return (None, Err(Error::Script(format!("function AST.run: env doesn't contain vars {}", env), pos))),
                     };
                     let pvars: HashableMap<RefTVal, RefTVal> = match env.get("parent") {
-                        Some(vs) => match vs.clone_out() {
-                            TVal { val: Value::Map(pvs), .. } => pvs,
+                        Some(vs) => match vs.clone_out().val {
+                            Value::Map(pvs) => pvs,
                             _ => return (None, Err(Error::Script(format!("function AST.run: env.parent is not a struct {}", vs), pos))),
                         },
                         None => hashmap!{}.into(),
