@@ -2,14 +2,16 @@ use std::collections::HashMap;
 use std::sync::Arc;
 use std::convert::TryFrom;
 
+use num_rational::BigRational;
+use num_traits::{One, Zero};
+
 use maplit::hashmap;
-use num::{BigRational, One};
 
 use crate::{Error, Env, Token, TokenType, Type, Value, TVal, RefTVal, Callable, FnArgs, FnReturn, AST, HashableMap};
 
 use super::_parse_margs;
 
-// Construct Macros: generic, macro, fn, struct, enum, trait
+// Construct Macros: generic, macro, fn, gen, struct, enum, trait
 
 pub fn r#generic(args: FnArgs) -> FnReturn {
     let (env, pos, tokens) = match _parse_margs("macro generic", args) {
@@ -124,7 +126,7 @@ pub fn r#macro(args: FnArgs) -> FnReturn {
     }.into()))
 }
 pub fn r#fn(args: FnArgs) -> FnReturn {
-    let (env, _pos, tokens) = match _parse_margs("macro fn", args) {
+    let (mut env, _pos, tokens) = match _parse_margs("macro fn", args) {
         Ok(t) => t,
         Err(m) => return (None, Err(m)),
     };
@@ -144,6 +146,8 @@ pub fn r#fn(args: FnArgs) -> FnReturn {
         body = tokens[4].clone();
     }
 
+    env.set("__is_gen", &Value::Number(BigRational::zero()).into());
+
     let fargs_val: Result<Vec<(String, Arc<Type>)>, Error> = fargs.data[1..fargs.data.len()-1].split(',')
         .filter_map(|vt| {
             if vt.is_empty() {
@@ -161,7 +165,7 @@ pub fn r#fn(args: FnArgs) -> FnReturn {
                                 } else {
                                     return Some(Err(Error::Script(format!("macro fn: parameter {}", m), Some(fargs.pos.clone()))));
                                 },
-                                _ => return Some(Err(Error::Script(format!("macro fn: parameter {}", m), Some(fargs.pos.clone()))))
+                                None => return Some(Err(Error::Script(format!("macro fn: parameter {}", m), Some(fargs.pos.clone()))))
                             },
                         }),
                     ))),
@@ -185,6 +189,80 @@ pub fn r#fn(args: FnArgs) -> FnReturn {
         body: Callable::AST(body_ast),
     }.into()))
 }
+pub fn r#gen(args: FnArgs) -> FnReturn {
+    let (mut env, _pos, tokens) = match _parse_margs("macro gen", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    let input = tokens[1].clone();
+    let fargs = tokens[3].clone();
+    let _ret;
+    let body;
+    if tokens.len() == 5 {
+        _ret = Token {
+            ttype: TokenType::Comment,
+            pos: fargs.pos.clone(),
+            data: "".into(),
+        };
+        body = tokens[4].clone();
+    } else {
+        _ret = tokens[5].clone();
+        body = tokens[6].clone();
+    }
+
+    let input_ast = match AST::parse(vec![input], &env) {
+        Ok(a) => a,
+        Err(m) => return (None, Err(m)),
+    };
+    let input_val = match input_ast.run(&env) {
+        (_, Ok(v)) => v,
+        (_, Err(m)) => return (None, Err(m)),
+    };
+    env.set("__gen_input_type", &input_val);
+    env.set("__is_gen", &Value::Number(BigRational::one()).into());
+
+    let fargs_val: Result<Vec<(String, Arc<Type>)>, Error> = fargs.data[1..fargs.data.len()-1].split(',')
+        .filter_map(|vt| {
+            if vt.is_empty() {
+                None
+            } else {
+                let mut vts = vt.split(':');
+                match (vts.next(), vts.next()) {
+                    (Some(vn), Some(t)) => Some(Ok((
+                        vn.trim().into(),
+                        Arc::new(match Type::try_from(t.trim()) {
+                            Ok(t) => t,
+                            Err(m) => match env.get(t.trim()) {
+                                Some(rv) => if let Value::Type(t, _) = rv.clone_out().val {
+                                    (*t).clone()
+                                } else {
+                                    return Some(Err(Error::Script(format!("macro gen: parameter {}", m), Some(fargs.pos.clone()))));
+                                },
+                                None => return Some(Err(Error::Script(format!("macro gen: parameter {}", m), Some(fargs.pos.clone()))))
+                            },
+                        }),
+                    ))),
+                    _ => Some(Err(Error::Script(format!("macro gen: parameter missing type {}", vt), Some(fargs.pos.clone())))),
+                }
+            }
+        }).collect();
+    let fargs_val = match fargs_val {
+        Ok(v) => v,
+        Err(m) => return (None, Err(m)),
+    };
+
+    let body_ast = match AST::parse(vec![body], &env) {
+        Ok(a) => a,
+        Err(m) => return (None, Err(m)),
+    };
+
+    (None, Ok(Value::Function {
+        args: Arc::new(fargs_val),
+        vars: Env::flatten(&env).vars.into(),
+        body: Callable::AST(body_ast),
+    }.into()))
+}
 
 pub fn r#struct(args: FnArgs) -> FnReturn {
     let (env, pos, tokens) = match _parse_margs("macro struct", args) {
@@ -226,7 +304,7 @@ pub fn r#struct(args: FnArgs) -> FnReturn {
         map.iter()
             .map(|(k, v)| {
                 if let Value::Type(t, _) = &v.clone_out().val {
-                    return (k.clone(), Arc::clone(&t));
+                    return (k.clone(), Arc::clone(t));
                 }
                 unreachable!()
             }).collect::<HashMap<String, Arc<Type>>>()
@@ -279,7 +357,7 @@ pub fn r#enum(args: FnArgs) -> FnReturn {
                                             if let AST::Value { token } = c {
                                                 let ct = env.get(&token.data);
                                                 if let Some(rv) = ct {
-                                                    if let TVal { val: Value::Type(ct, _), .. } = rv.clone_out() {
+                                                    if let Value::Type(ct, _) = rv.clone_out().val {
                                                         es.push((token.data.clone(), Arc::clone(&ct)));
                                                     }
                                                 } else {
@@ -311,7 +389,7 @@ pub fn r#enum(args: FnArgs) -> FnReturn {
         .collect();
     let enums = Arc::new(enums);
     for v in map.map.values_mut() {
-        let e: Box<(String, Vec<RefTVal>)> = if let TVal { val: Value::Enum(_, ref e), .. } = *v.try_lock().unwrap() {
+        let e: Box<(String, Vec<RefTVal>)> = if let Value::Enum(_, ref e) = v.try_lock().unwrap().val {
             e.clone()
         } else {
             unreachable!();
@@ -366,7 +444,7 @@ pub fn r#trait(args: FnArgs) -> FnReturn {
         map.iter()
             .map(|(k, v)| {
                 if let Value::Type(t, _) = &v.clone_out().val {
-                    return (k.clone(), Arc::clone(&t));
+                    return (k.clone(), Arc::clone(t));
                 }
                 unreachable!()
             }).collect::<HashMap<String, Arc<Type>>>()
