@@ -1,10 +1,13 @@
-use num::{BigRational, One, Zero};
+use num_rational::BigRational;
+use num_traits::{One, Zero};
 
-use crate::{AST, Env, Error, FnArgs, FnReturn, TVal, TokenType, Value, match_pat};
+use maplit::hashmap;
+
+use crate::{AST, Env, Error, FnArgs, FnReturn, RefTVal, TokenType, Value, match_pat};
 
 use super::_parse_margs;
 
-// Control Macros: if, match, loop, return
+// Control Macros: if, match, loop, return, input, yield
 
 pub fn r#if(args: FnArgs) -> FnReturn {
     let (env, pos, tokens) = match _parse_margs("macro if", args) {
@@ -32,18 +35,25 @@ pub fn r#if(args: FnArgs) -> FnReturn {
         Ok(a) => a,
         Err(m) => return (None, Err(m)),
     };
-    let (_, cond_val) = cond_ast.run(&env);
+    let (cond_vars, cond_val) = cond_ast.run(&env);
 
+    let mut nenv = Env::child(&env);
     if let Ok(crv) = &cond_val {
-        if let TVal { val: Value::Number(n), .. } = crv.clone_out() {
+        if let Value::Number(n) = crv.clone_out().val {
             if n == BigRational::one() {
                 // Cond succeeds, run body
                 let body_ast = match AST::parse(vec![body.clone()], &env) {
                     Ok(a) => a,
                     Err(m) => return (None, Err(m)),
                 };
-                let (_, body_val) = body_ast.run(&env);
-                return (Some(env.vars), body_val);
+                if let Some(cond_vars) = cond_vars {
+                    nenv.update(cond_vars);
+                }
+                let (vars, body_val) = body_ast.run(&nenv);
+                if let Some(vars) = vars {
+                    nenv.update(vars);
+                }
+                return (Some(nenv.vars), body_val);
             } else if n == BigRational::zero() {
                 // Cond fails, run else blocks
                 if let Some(el) = tokens.get(i+1) {
@@ -58,12 +68,15 @@ pub fn r#if(args: FnArgs) -> FnReturn {
                                 });
                             }
                             // Lone else
-                            let else_ast = match AST::parse(vec![ifblock.clone()], &env) {
+                            let else_ast = match AST::parse(vec![ifblock.clone()], &nenv) {
                                 Ok(a) => a,
                                 Err(m) => return (None, Err(m)),
                             };
-                            let (_, else_val) = else_ast.run(&env);
-                            return (Some(env.vars), else_val);
+                            let (vars, else_val) = else_ast.run(&env);
+                            if let Some(vars) = vars {
+                                nenv.update(vars);
+                            }
+                            return (Some(nenv.vars), else_val);
                         }
                         return (None, Err(Error::Script("macro if: invalid else statement".into(), pos)));
                     }
@@ -118,14 +131,14 @@ pub fn r#match(args: FnArgs) -> FnReturn {
                                 }
                                 return rhs.run(&nenv);
                             },
-                            Err(_m) => {
-                                // eprintln!("{}", _m);
+                            Err(m) => {
+                                log::trace!("{}", m);
                             },
                         }
                     },
                     AST::Container { token, .. } if token.data.starts_with("//") => {},
                     _ => {
-                        // eprintln!("mitem {}\n\nc {:?}\n", mitem_val, c);
+                        log::trace!("mitem {}\n\nc {:?}\n", mitem_val, c);
                         return (None, Err(Error::Script(format!("macro match: invalid row {}", c.to_string_lossy()), pos)))
                     },
                 }
@@ -155,16 +168,21 @@ pub fn r#loop(args: FnArgs) -> FnReturn {
             Ok(_) => if let Some(vars) = vars {
                 nenv.update(vars);
             },
-            Err(Error::Control(s, v)) if s == "break" => match v {
-                Some(v) => {
-                    if let Value::Enum(_, e) = v.val {
-                        if let Some(v) = e.1.get(0) {
-                            return (None, Ok(v.clone()));
+            Err(Error::Control(s, v)) if s == "break" => {
+                if let Some(vars) = vars {
+                    nenv.update(vars);
+                }
+                match v {
+                    Some(v) => {
+                        if let Value::Enum(_, e) = v.val {
+                            if let Some(v) = e.1.get(0) {
+                                return (Some(nenv.vars), Ok(Value::from_option(&Some(v.clone())).into()));
+                            }
                         }
-                    }
-                    break;
-                },
-                None => break,
+                        break;
+                    },
+                    None => break,
+                }
             },
             Err(Error::Control(s, _)) if s == "continue" => {
                 if let Some(vars) = vars {
@@ -176,7 +194,7 @@ pub fn r#loop(args: FnArgs) -> FnReturn {
         }
     }
 
-    (None, Ok(Value::none().into()))
+    (Some(nenv.vars), Ok(Value::from_option(&None).into()))
 }
 pub fn r#return(args: FnArgs) -> FnReturn {
     let (env, _pos, tokens) = match _parse_margs("macro return", args) {
@@ -199,3 +217,61 @@ pub fn r#return(args: FnArgs) -> FnReturn {
         Err(m) => (None, Err(m)),
     }
 }
+pub fn r#input(args: FnArgs) -> FnReturn {
+    let (env, pos, tokens) = match _parse_margs("macro input", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    let varname = &tokens[1].data;
+
+    if let Some(th) = env.get("__current_thread") {
+        if let Value::Thread(th) = th.clone_out().val {
+            if let Ok(ginput) = th.ginput.lock() {
+                if let Some(rx) = &ginput.1 {
+                    if let Ok(val) = rx.recv() {
+                        return (hashmap!{ varname.into() => val.into() }.into(), Ok(Value::Number(BigRational::one()).into()));
+                    }
+                }
+            }
+            return (None, Ok(Value::Number(BigRational::zero()).into()));
+        }
+    }
+
+    (None, Err(Error::Script("macro input: not inside a generator".into(), pos)))
+}
+pub fn r#yield(args: FnArgs) -> FnReturn {
+    let (env, pos, tokens) = match _parse_margs("macro yield", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    let val: RefTVal;
+    if tokens.len() == 1 {
+        val = Value::none().into();
+    } else {
+        let ast = match AST::parse(tokens[1..].iter().cloned(), &env) {
+            Ok(a) => a,
+            Err(m) => return (None, Err(m)),
+        };
+        val = match ast.run(&env) {
+            (_, Ok(v)) => v,
+            (_, Err(m)) => return (None, Err(m)),
+        };
+    }
+
+    if let Some(th) = env.get("__current_thread") {
+        if let Value::Thread(th) = th.clone_out().val {
+            if let Ok(goutput) = th.goutput.lock() {
+                if let Some(tx) = &goutput.0 {
+                    match tx.send(val.clone_out()) {
+                        Ok(()) => return (None, Ok(Value::none().into())),
+                        Err(m) => return (None, Err(Error::Script(format!("macro yield: failed to send value: {}", m), pos))),
+                    }
+                }
+            }
+        }
+    }
+
+    (None, Err(Error::Script("macro yield: not inside a generator".into(), pos)))
+}
