@@ -22,7 +22,7 @@ pub use types::*;
 mod composite;
 pub use composite::*;
 
-pub fn init<'a>(env: &mut Env<'a>) {
+pub fn init(env: &mut Env<'_>) {
     env.update(hashmap!{
         ////////////
         // Macros //
@@ -30,17 +30,21 @@ pub fn init<'a>(env: &mut Env<'a>) {
         // Base
         "use".into() => _macro_init(r#use),
         "let".into() => _macro_init(r#let),
+        "mod".into() => _macro_init(r#mod),
 
         // Control
         "if".into() => _macro_init(r#if),
         "match".into() => _macro_init(r#match),
         "loop".into() => _macro_init(r#loop),
         "return".into() => _macro_init(r#return),
+        "input".into() => _macro_init(r#input),
+        "yield".into() => _macro_init(r#yield),
 
         // Construct
         "generic".into() => _macro_init(r#generic),
         "macro".into() => _macro_init(r#macro),
         "fn".into() => _macro_init(r#fn),
+        "gen".into() => _macro_init(r#gen),
         "struct".into() => _macro_init(r#struct),
         "enum".into() => _macro_init(r#enum),
         "trait".into() => _macro_init(r#trait),
@@ -51,6 +55,7 @@ pub fn init<'a>(env: &mut Env<'a>) {
         // Functions
         "eval".into() => eval_init(),
         "exit".into() => exit_init(),
+        "type".into() => ftype_init(),
 
         // Base Types
         "Type".into() => type_init(),
@@ -88,10 +93,10 @@ pub fn _macro_init(func: fn(args: FnArgs) -> FnReturn) -> RefTVal {
 pub fn _macro_args() -> Arc<Vec<(String, Arc<Type>)>> {
     Arc::clone(&MACRO_ARGS)
 }
-pub fn _parse_margs<'a>(name: &str, args: FnArgs) -> Result<(Env<'a>, Option<Pos>, Vec<Token>), Error> {
+pub fn _parse_margs(name: &str, args: FnArgs) -> Result<(Env<'_>, Option<Pos>, Vec<Token>), Error> {
     match args {
         FnArgs::Macro { vars, pos, tokens } => Ok((Env::from(vars), pos, tokens)),
-        _ => Err(Error::Script(format!("{}: invalid FnArgs", name), None)),
+        FnArgs::Normal { .. } => Err(Error::Script(format!("{}: invalid FnArgs", name), None)),
     }
 }
 pub fn _parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<Pos>, RefTVal), Error> {
@@ -99,6 +104,6 @@ pub fn _parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<Po
         FnArgs::Normal { this, pos, args } => {
             Ok((this, pos, args))
         },
-        _ => Err(Error::Script(format!("{}: invalid FnArgs", name), None)),
+        FnArgs::Macro { .. } => Err(Error::Script(format!("{}: invalid FnArgs", name), None)),
     }
 }
