@@ -1,6 +1,7 @@
 use std::sync::Arc;
 
-use num::{BigRational, FromPrimitive, ToPrimitive, One, Zero};
+use num_rational::BigRational;
+use num_traits::{FromPrimitive, ToPrimitive, One, Zero};
 
 use maplit::hashmap;
 
@@ -308,6 +309,31 @@ pub fn list_init() -> RefTVal {
             vars: hashmap!{}.into(),
             body: Callable::Native(list_sort_by_key),
         }.into(),
+
+        "join".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::list()),
+                ("s".into(), Type::string()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(list_join),
+        }.into(),
+        "slice".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::list()),
+                ("l".into(), Type::list()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(list_slice),
+        }.into(),
+        "find".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::list()),
+                ("val".into(), Type::any()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(list_find),
+        }.into(),
     }.into())).into();
     vt.attr.insert("impls".into(), Value::Map(hashmap!{}.into()).into());
     vt.into()
@@ -320,7 +346,7 @@ fn list_parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<Po
     };
     if let Value::List(ref targs) = args.clone_out().val {
         if let Some(rv0) = targs.get(0) {
-            if let TVal { val: Value::List(l), .. } = rv0.clone_out() {
+            if let Value::List(l) = rv0.clone_out().val {
                 return Ok((this, pos, Some(l), targs.clone()));
             }
         }
@@ -400,9 +426,11 @@ pub fn list_remove(args: FnArgs) -> FnReturn {
         if let Value::List(ref mut l) = rv0.try_lock().unwrap().val {
             if let Some(idx) = args.get(1) {
                 if let Value::Number(idx) = idx.clone_out().val {
-                    let idx = idx.to_usize().unwrap();
-                    if idx < l.len() {
-                        return (None, Ok(Value::from_option(&Some(l.remove(idx))).into()));
+                    if idx >= BigRational::zero() {
+                        let idx = idx.to_usize().unwrap();
+                        if idx < l.len() {
+                            return (None, Ok(Value::from_option(&Some(l.remove(idx))).into()));
+                        }
                     }
                     return (None, Ok(Value::from_option(&None).into()));
                 }
@@ -644,10 +672,8 @@ pub fn list_split_first(args: FnArgs) -> FnReturn {
     };
 
     if let Some(l) = l {
-        let splf: Option<RefTVal> = match l.split_first() {
-            Some((f, r)) => Some(Value::List(vec![f.to_owned(), Value::List(r.to_vec()).into()]).into()),
-            None => None,
-        };
+        let splf: Option<RefTVal> = l.split_first()
+            .map(|(f, r)| Value::List(vec![f.clone(), Value::List(r.to_vec()).into()]).into());
         return (None, Ok(Value::from_option(&splf).into()));
     }
 
@@ -660,10 +686,8 @@ pub fn list_split_last(args: FnArgs) -> FnReturn {
     };
 
     if let Some(l) = l {
-        let spll: Option<RefTVal> = match l.split_last() {
-            Some((l, r)) => Some(Value::List(vec![l.to_owned(), Value::List(r.to_vec()).into()]).into()),
-            None => None,
-        };
+        let spll: Option<RefTVal> = l.split_last()
+            .map(|(l, r)| Value::List(vec![l.clone(), Value::List(r.to_vec()).into()]).into());
         return (None, Ok(Value::from_option(&spll).into()));
     }
 
@@ -1098,3 +1122,73 @@ pub fn list_sort_by_key(args: FnArgs) -> FnReturn {
 
     (None, Err(Error::Script(format!("function List.sort_by_key: expected args [List, Function], got {:?}", args), pos)))
 }
+
+pub fn list_join(args: FnArgs) -> FnReturn {
+    let (_this, pos, l, args) = match list_parse_fargs("function List.join", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(l) = l {
+        if let Some(sep) = args.get(1) {
+            if let Value::String(sep) = sep.clone_out().val {
+                let mut v: Vec<String> = vec![];
+                for s in l {
+                    if let Value::String(s) = s.clone_out().val {
+                        v.push(s);
+                    } else {
+                        return (None, Err(Error::Script(format!("function List.join: got non-string: {}", s), pos)));
+                    }
+                }
+
+                return (None, Ok(Value::String(v.join(&sep)).into()));
+            }
+        }
+    }
+
+    (None, Err(Error::Script(format!("function List.join: expected args [List, String], got {:?}", args), pos)))
+}
+pub fn list_slice(args: FnArgs) -> FnReturn {
+    let (_this, pos, l, args) = match list_parse_fargs("function List.slice", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(l) = l {
+        if let Some(idxs) = args.get(1) {
+            if let Value::List(idxs) = idxs.clone_out().val {
+                let mut vs: Vec<RefTVal> = vec![];
+                for (i, v) in l.iter().enumerate() {
+                    if idxs.contains(&Value::Number(BigRational::from_usize(i).unwrap()).into()) {
+                        vs.push(v.clone());
+                    }
+                }
+                return (None, Ok(Value::List(vs).into()));
+            }
+        }
+    }
+
+    (None, Err(Error::Script(format!("function List.slice: expected args [List, List], got {:?}", args), pos)))
+}
+pub fn list_find(args: FnArgs) -> FnReturn {
+    let (_this, pos, l, args) = match list_parse_fargs("function List.find", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(l) = l {
+        if let Some(val) = args.get(1) {
+            if l.contains(val) {
+                for (i, v) in l.iter().enumerate() {
+                    if v == val {
+                        return (None, Ok(Value::from_option(&Some(Value::Number(BigRational::from_usize(i).unwrap()).into())).into()));
+                    }
+                }
+            } else {
+                return (None, Ok(Value::from_option(&None).into()));
+            }
+        }
+    }
+
+    (None, Err(Error::Script(format!("function List.find: expected args [List, Any], got {:?}", args), pos)))
+}
