@@ -1,12 +1,16 @@
 use std::sync::Arc;
 use std::str::FromStr;
+use std::cmp::Ordering;
 
-use lazy_static::lazy_static;
-use num::{BigInt, BigRational, FromPrimitive, One, Signed, ToPrimitive};
+use num_rational::BigRational;
+use num_bigint::BigInt;
+use num_traits::{FromPrimitive, One, Signed, ToPrimitive};
 
 use maplit::hashmap;
 
-use crate::{Callable, Error, FnArgs, FnReturn, RefTVal, TVal, Type, Value, Pos};
+use lazy_static::lazy_static;
+
+use crate::{Callable, Error, FnArgs, FnReturn, RefTVal, Type, Value, Pos};
 
 use super::_parse_fargs;
 
@@ -287,6 +291,15 @@ pub fn number_init() -> RefTVal {
             vars: hashmap!{}.into(),
             body: Callable::Native(number_reverse_bits),
         }.into(),
+
+        "cmp".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::number()),
+                ("other".into(), Type::number()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(number_cmp),
+        }.into(),
     }.into())).into()
 }
 
@@ -297,7 +310,7 @@ fn number_parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<
     };
     if let Value::List(ref targs) = args.clone_out().val {
         if let Some(rv0) = targs.get(0) {
-            if let TVal { val: Value::Number(x), .. } = rv0.clone_out() {
+            if let Value::Number(x) = rv0.clone_out().val {
                 return Ok((this, pos, Some(x), targs.clone()));
             }
         }
@@ -803,3 +816,29 @@ pub fn number_reverse_bits(args: FnArgs) -> FnReturn {
 
     (None, Err(Error::Script(format!("function Number.reverse_bits: expected args [Number], got {:?}", args), pos)))
 }
+
+pub fn number_cmp(args: FnArgs) -> FnReturn {
+    let (_this, pos, x, args) = match number_parse_fargs("function Number.cmp", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(x) = x {
+        if let Some(orv) = args.get(1) {
+            if let Value::Number(other) = orv.clone_out().val {
+                let es = Arc::new(vec![
+                    ("Lesser".into(), Type::none()),
+                    ("Greater".into(), Type::none()),
+                    ("Equal".into(), Type::none()),
+                ]);
+                match x.cmp(&other) {
+                    Ordering::Less => return (None, Ok(Value::Enum(es, Box::new(("Lesser".into(), vec![]))).into())),
+                    Ordering::Greater => return (None, Ok(Value::Enum(es, Box::new(("Greater".into(), vec![]))).into())),
+                    Ordering::Equal => return (None, Ok(Value::Enum(es, Box::new(("Equal".into(), vec![]))).into())),
+                }
+            }
+        }
+    }
+
+    (None, Err(Error::Script(format!("function Number.cmp: expected args [Number, Number], got {:?}", args), pos)))
+}
