@@ -1,15 +1,24 @@
 use std::sync::Arc;
 
-use num::{BigRational, One, Zero, ToPrimitive, FromPrimitive};
+use num_rational::BigRational;
+use num_traits::{One, Zero, ToPrimitive, FromPrimitive};
 
 use maplit::hashmap;
 
-use crate::{Callable, Error, FnArgs, FnReturn, RefTVal, TVal, Type, Value, Pos, Env};
+use crate::parsing::parse_number;
+use crate::{Callable, Env, Error, FnArgs, FnReturn, Pos, RefTVal, TVal, Type, Value, parsing::unescape};
 
 use super::_parse_fargs;
 
 pub fn string_init() -> RefTVal {
     let mut vt: TVal = Value::Type(Type::string(), Arc::new(hashmap!{
+        "to_bytes".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::string()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(string_to_bytes),
+        }.into(),
         "push".into() => Value::Function {
             args: Arc::new(vec![
                 ("self".into(), Type::string()),
@@ -292,6 +301,21 @@ pub fn string_init() -> RefTVal {
             vars: hashmap!{}.into(),
             body: Callable::Native(string_to_uppercase),
         }.into(),
+
+        "escape".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::string()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(string_escape),
+        }.into(),
+        "unescape".into() => Value::Function {
+            args: Arc::new(vec![
+                ("self".into(), Type::string()),
+            ]),
+            vars: hashmap!{}.into(),
+            body: Callable::Native(string_unescape),
+        }.into(),
     }.into())).into();
     vt.attr.insert("impls".into(), Value::Map(hashmap!{}.into()).into());
     vt.into()
@@ -304,7 +328,7 @@ fn string_parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<
     };
     if let Value::List(ref targs) = args.clone_out().val {
         if let Some(rv0) = targs.get(0) {
-            if let TVal { val: Value::String(s), .. } = rv0.clone_out() {
+            if let Value::String(s) = rv0.clone_out().val {
                 return Ok((this, pos, Some(s), targs.clone()));
             }
         }
@@ -312,6 +336,21 @@ fn string_parse_fargs(name: &str, args: FnArgs) -> Result<(Box<RefTVal>, Option<
     Ok((this, pos, None, vec![args]))
 }
 
+pub fn string_to_bytes(args: FnArgs) -> FnReturn {
+    let (_this, pos, s, args) = match string_parse_fargs("function String.to_bytes", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(s) = s {
+        let l: Vec<RefTVal> = s.as_bytes().iter()
+            .map(|b| Value::Number(BigRational::from_u8(*b).unwrap()).into())
+            .collect();
+        return (None, Ok(Value::List(l).into()));
+    }
+
+    (None, Err(Error::Script(format!("function String.to_bytes: expected args [String], got {:?}", args), pos)))
+}
 pub fn string_push(args: FnArgs) -> FnReturn {
     let (_this, pos, _s, args) = match string_parse_fargs("function String.push", args) {
         Ok(t) => t,
@@ -319,9 +358,9 @@ pub fn string_push(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
-                if let TVal { val: Value::String(s2), .. } = rv1.clone_out() {
+                if let Value::String(s2) = rv1.clone_out().val {
                     s1.push_str(&s2);
                     return (None, Ok(Value::none().into()));
                 }
@@ -338,9 +377,9 @@ pub fn string_truncate(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
-                if let TVal { val: Value::Number(l), .. } = rv1.clone_out() {
+                if let Value::Number(l) = rv1.clone_out().val {
                     s1.truncate(l.to_usize().unwrap());
                     return (None, Ok(Value::none().into()));
                 }
@@ -357,9 +396,9 @@ pub fn string_pop(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
-                if let TVal { val: Value::Number(n), .. } = rv1.clone_out() {
+                if let Value::Number(n) = rv1.clone_out().val {
                     return (None, Ok(Value::String(s1.split_off(s1.len() - n.to_usize().unwrap())).into()));
                 }
             }
@@ -375,9 +414,9 @@ pub fn string_remove(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
-                if let TVal { val: Value::Number(idx), .. } = rv1.clone_out() {
+                if let Value::Number(idx) = rv1.clone_out().val {
                     let idx = idx.to_usize().unwrap();
                     return if idx < s.len() {
                         (None, Ok(Value::from_option(&Some((&*s.remove(idx).to_string()).into())).into()))
@@ -398,9 +437,9 @@ pub fn string_retain(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
-                if let TVal { val: Value::Function { vars, body, .. }, .. } = rv1.clone_out() {
+                if let Value::Function { vars, body, .. } = rv1.clone_out().val {
                     s1.retain(|c: char| -> bool {
                         let c: RefTVal = Value::String(c.to_string()).into();
                         let mut vars = vars.clone();
@@ -435,7 +474,7 @@ pub fn string_insert(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
                 if let Value::Number(n) = rv1.clone_out().val {
                     if let Some(rv2) = args.get(2) {
@@ -488,7 +527,7 @@ pub fn string_split_off(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             if let Some(rv1) = args.get(1) {
                 if let Value::Number(n) = rv1.clone_out().val {
                     return (None, Ok(Value::String(s1.split_off(n.to_usize().unwrap())).into()));
@@ -506,7 +545,7 @@ pub fn string_clear(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val {
             s1.clear();
             return (None, Ok(Value::none().into()));
         }
@@ -521,7 +560,7 @@ pub fn string_replace_range(args: FnArgs) -> FnReturn {
     };
 
     if let Some(rv0) = args.get(0) {
-        if let TVal { val: Value::String(ref mut s1), .. } = *rv0.try_lock().unwrap() {
+        if let Value::String(ref mut s1) = rv0.try_lock().unwrap().val  {
             if let Some(rv1) = args.get(1) {
                 if let Value::List(l) = rv1.clone_out().val {
                     if let Some(rv2) = args.get(2) {
@@ -909,8 +948,8 @@ pub fn string_parse(args: FnArgs) -> FnReturn {
         Err(m) => return (None, Err(m)),
     };
 
-    if let Some(_s) = s {
-        // TODO
+    if let Some(s) = s {
+        return parse_number(&s, pos);
     }
 
     (None, Err(Error::Script(format!("function String.parse: expected args [String], got {:?}", args), pos)))
@@ -983,3 +1022,28 @@ pub fn string_to_uppercase(args: FnArgs) -> FnReturn {
 
     (None, Err(Error::Script(format!("function String.to_uppercase: expected args [String], got {:?}", args), pos)))
 }
+
+pub fn string_escape(args: FnArgs) -> FnReturn {
+    let (_this, pos, s, args) = match string_parse_fargs("function String.escape", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(s) = s {
+        return (None, Ok(Value::String(s.escape_default().to_string()).into()));
+    }
+
+    (None, Err(Error::Script(format!("function String.escape: expected args [String], got {:?}", args), pos)))
+}
+pub fn string_unescape(args: FnArgs) -> FnReturn {
+    let (_this, pos, s, args) = match string_parse_fargs("function String.unescape", args) {
+        Ok(t) => t,
+        Err(m) => return (None, Err(m)),
+    };
+
+    if let Some(s) = s {
+        return (None, Ok(Value::String(unescape(&s)).into()));
+    }
+
+    (None, Err(Error::Script(format!("function String.unescape: expected args [String], got {:?}", args), pos)))
+}
