@@ -3,7 +3,7 @@ use std::sync::Arc;
 
 use maplit::hashmap;
 
-use crate::{TVal, RefTVal, Value, FnArgs, FnReturn, Callable, Type, Error, umcore::_parse_fargs};
+use crate::{RefTVal, Value, FnArgs, FnReturn, Callable, Type, Error, umcore::_parse_fargs};
 
 pub fn init(vars: &mut HashMap<String, RefTVal>) {
     vars.extend(hashmap!{
@@ -29,10 +29,12 @@ pub fn format(args: FnArgs) -> FnReturn {
 
     if let Value::List(ref args) = args.clone_out().val {
         if let Some(rv0) = args.get(0) {
-            if let TVal { val: Value::String(f), .. } = rv0.clone_out() {
+            if let Value::String(f) = rv0.clone_out().val {
                 let ps = args[1..].iter()
-                    .map(|a| a.clone_out().val.to_string())
-                    .collect::<Vec<String>>();
+                    .map(|a| match a.clone_out().val {
+                        Value::Reference(rv) => rv.clone_out().val.to_string(),
+                        v => v.to_string(),
+                    }).collect::<Vec<String>>();
                 let mut s = f;
                 for p in ps {
                     s = s.replacen("{}", &p, 1);
